@@ -1,5 +1,5 @@
 /**
- * $Id: GraphEditor.java,v 1.5 2010-08-10 11:43:36 david Exp $
+ * $Id: GraphEditor.java,v 1.8 2011-02-14 15:45:58 gaudenz Exp $
  * Copyright (c) 2006-2010, Gaudenz Alder, David Benson
  */
 package com.mxgraph.examples.swing;
@@ -29,10 +29,10 @@ import com.mxgraph.swing.util.mxGraphTransferable;
 import com.mxgraph.util.mxConstants;
 import com.mxgraph.util.mxEvent;
 import com.mxgraph.util.mxEventObject;
+import com.mxgraph.util.mxEventSource.mxIEventListener;
 import com.mxgraph.util.mxPoint;
 import com.mxgraph.util.mxResources;
 import com.mxgraph.util.mxUtils;
-import com.mxgraph.util.mxEventSource.mxIEventListener;
 import com.mxgraph.view.mxCellState;
 import com.mxgraph.view.mxGraph;
 
@@ -107,6 +107,22 @@ public class GraphEditor extends BasicGraphEditor
 								GraphEditor.class
 										.getResource("/com/mxgraph/examples/swing/images/swimlane.png")),
 						"swimlane", 280, 280, "Container");
+		shapesPalette
+				.addTemplate(
+						"Icon",
+						new ImageIcon(
+								GraphEditor.class
+										.getResource("/com/mxgraph/examples/swing/images/rounded.png")),
+						"icon;image=/com/mxgraph/examples/swing/images/wrench.png",
+						70, 70, "Icon");
+		shapesPalette
+				.addTemplate(
+						"Label",
+						new ImageIcon(
+								GraphEditor.class
+										.getResource("/com/mxgraph/examples/swing/images/rounded.png")),
+						"label;image=/com/mxgraph/examples/swing/images/gear.png",
+						130, 50, "Label");
 		shapesPalette
 				.addTemplate(
 						"Rectangle",
@@ -449,8 +465,8 @@ public class GraphEditor extends BasicGraphEditor
 			codec.decode(doc.getDocumentElement(), graph.getStylesheet());
 
 			// Sets the background to white
-			getViewport().setOpaque(false);
-			setBackground(Color.WHITE);
+			getViewport().setOpaque(true);
+			getViewport().setBackground(Color.WHITE);
 		}
 
 		/**
@@ -648,6 +664,8 @@ public class GraphEditor extends BasicGraphEditor
 		}
 
 		mxConstants.SHADOW_COLOR = Color.LIGHT_GRAY;
+		mxConstants.W3C_SHADOWCOLOR = "#D3D3D3";
+		
 		GraphEditor editor = new GraphEditor();
 		editor.createFrame(new EditorMenuBar(editor)).setVisible(true);
 	}
