@@ -1,5 +1,5 @@
 /**
- * $Id: mxCellCodec.java,v 1.13 2010-04-30 07:28:15 gaudenz Exp $
+ * $Id: mxCellCodec.java,v 1.14 2011-07-07 10:12:24 gaudenz Exp $
  * Copyright (c) 2006, Gaudenz Alder
  */
 package com.mxgraph.io;
@@ -103,8 +103,20 @@ public class mxCellCodec extends mxObjectCodec
 		{
 			mxCell cell = (mxCell) obj;
 			String classname = getName();
+			String nodeName = node.getNodeName();
+			
+			// Handles aliased names
+			if (!nodeName.equals(classname))
+			{
+				String tmp = mxCodecRegistry.aliases.get(nodeName);
+				
+				if (tmp != null)
+				{
+					nodeName = tmp;
+				}
+			}
 
-			if (!node.getNodeName().equals(classname))
+			if (!nodeName.equals(classname))
 			{
 				// Passes the inner graphical annotation node to the
 				// object codec for further processing of the cell.
