@@ -1,5 +1,5 @@
 /**
- * $Id: mxCodec.java,v 1.28 2010-03-02 12:42:58 gaudenz Exp $
+ * $Id: mxCodec.java,v 1.31 2010-10-18 10:45:50 gaudenz Exp $
  * Copyright (c) 2006, Gaudenz Alder
  */
 package com.mxgraph.io;
@@ -63,6 +63,24 @@ public class mxCodec
 		this.document = document;
 	}
 
+	/**
+	 * Returns the owner document of the codec.
+	 * 
+	 * @return Returns the owner document.
+	 */
+	public Document getDocument()
+	{
+		return document;
+	}
+
+	/**
+	 * Sets the owner document of the codec.
+	 */
+	public void setDocument(Document value)
+	{
+		document = value;
+	}
+
 	/**
 	 * Returns if default values of member variables should be encoded.
 	 */
@@ -87,16 +105,6 @@ public class mxCodec
 		return objects;
 	}
 
-	/**
-	 * Returns the owner document of the codec.
-	 * 
-	 * @return Returns the owner document.
-	 */
-	public Document getDocument()
-	{
-		return document;
-	}
-
 	/**
 	 * Assoiates the given object with the given ID.
 	 * 
@@ -411,31 +419,42 @@ public class mxCodec
 
 			if (restoreStructures)
 			{
-				mxICell parent = cell.getParent();
-
-				if (parent != null)
-				{
-					parent.insert(cell);
-				}
-
-				mxICell source = cell.getTerminal(true);
-
-				if (source != null)
-				{
-					source.insertEdge(cell, true);
-				}
-
-				mxICell target = cell.getTerminal(false);
-
-				if (target != null)
-				{
-					target.insertEdge(cell, false);
-				}
+				insertIntoGraph(cell);
 			}
 		}
 
 		return cell;
 	}
+	
+	/**
+	 * Inserts the given cell into its parent and terminal cells.
+	 */
+	public void insertIntoGraph(mxICell cell)
+	{
+		mxICell parent = cell.getParent();
+		mxICell source = cell.getTerminal(true);
+		mxICell target = cell.getTerminal(false);
+
+		// Fixes possible inconsistencies during insert into graph
+		cell.setTerminal(null, false);
+		cell.setTerminal(null, true);
+		cell.setParent(null);
+		
+		if (parent != null)
+		{
+			parent.insert(cell);
+		}
+
+		if (source != null)
+		{
+			source.insertEdge(cell, true);
+		}
+
+		if (target != null)
+		{
+			target.insertEdge(cell, false);
+		}
+	}
 
 	/**
 	 * Sets the attribute on the specified node to value. This is a
