@@ -1,6 +1,6 @@
 /**
- * $Id: mxModelCodec.java,v 1.6 2010-04-30 07:28:15 gaudenz Exp $
- * Copyright (c) 2006, Gaudenz Alder
+ * $Id: mxModelCodec.java,v 1.8 2010-11-23 08:46:41 gaudenz Exp $
+ * Copyright (c) 2006-2010, Gaudenz Alder, David Benson
  */
 package com.mxgraph.io;
 
@@ -14,7 +14,7 @@ import com.mxgraph.model.mxICell;
 
 /**
  * Codec for mxGraphModels. This class is created and registered
- * dynamically at load time and used implicitely via mxCodec
+ * dynamically at load time and used implicitly via mxCodec
  * and the mxCodecRegistry.
  */
 public class mxModelCodec extends mxObjectCodec
@@ -46,26 +46,19 @@ public class mxModelCodec extends mxObjectCodec
 	}
 
 	/**
-	 * Encode the given mxGraphModel by writing a (flat) XML sequence
+	 * Encodes the given mxGraphModel by writing a (flat) XML sequence
 	 * of cell nodes as produced by the mxCellCodec. The sequence is
 	 * wrapped-up in a node with the name root.
 	 */
-	public Node encode(mxCodec enc, Object obj)
+	protected void encodeObject(mxCodec enc, Object obj, Node node)
 	{
-		Node node = null;
-
 		if (obj instanceof mxGraphModel)
 		{
-			mxGraphModel model = (mxGraphModel) obj;
-
-			node = enc.document.createElement(getName());
 			Node rootNode = enc.document.createElement("root");
-
+			mxGraphModel model = (mxGraphModel) obj;
 			enc.encodeCell((mxICell) model.getRoot(), rootNode, true);
 			node.appendChild(rootNode);
 		}
-
-		return node;
 	}
 
 	/**
