@@ -1,5 +1,5 @@
 /**
- * $Id: mxObjectCodec.java,v 1.34 2010-08-31 11:23:12 gaudenz Exp $
+ * $Id: mxObjectCodec.java,v 1.36 2011-01-31 13:23:55 gaudenz Exp $
  * Copyright (c) 2006, Gaudenz Alder
  */
 package com.mxgraph.io;
@@ -781,8 +781,9 @@ public class mxObjectCodec
 
 			if (field.getType() == Boolean.class)
 			{
-				value = new Boolean(value.equals("1")
-						|| String.valueOf(value).equalsIgnoreCase("true"));
+				value = (value.equals("1") || String.valueOf(value)
+						.equalsIgnoreCase("true")) ? Boolean.TRUE
+						: Boolean.FALSE;
 			}
 
 			field.set(obj, value);
@@ -801,8 +802,8 @@ public class mxObjectCodec
 					if (type.isArray() && value instanceof Collection)
 					{
 						Collection<?> coll = (Collection<?>) value;
-						value = coll.toArray((Object[]) Array.newInstance(type
-								.getComponentType(), coll.size()));
+						value = coll.toArray((Object[]) Array.newInstance(
+								type.getComponentType(), coll.size()));
 					}
 
 					method.invoke(obj, new Object[] { value });
@@ -1023,8 +1024,8 @@ public class mxObjectCodec
 
 		if (fieldname == null || !isExcluded(obj, fieldname, child, false))
 		{
+			Object template = getFieldTemplate(obj, fieldname, child);
 			Object value = null;
-			Object template = getFieldValue(obj, fieldname);
 
 			if (child.getNodeName().equals("add"))
 			{
@@ -1037,38 +1038,67 @@ public class mxObjectCodec
 			}
 			else
 			{
-				// Arrays are replaced completely
-				if (template != null && template.getClass().isArray())
-				{
-					template = null;
-				}
-				// Collections are cleared
-				else if (template instanceof Collection)
-				{
-					((Collection<?>) template).clear();
-				}
-
 				value = dec.decode(child, template);
 				// System.out.println("Decoded " + child.getNodeName() + "."
 				// + fieldname + "=" + value);
 			}
 
-			if (value != null && !value.equals(template))
+			addObjectValue(obj, fieldname, value, template);
+		}
+	}
+
+	/**
+	 * Returns the template instance for the given field. This returns the
+	 * value of the field, null if the value is an array or an empty collection
+	 * if the value is a collection. The value is then used to populate the
+	 * field for a new instance. For strongly typed languages it may be
+	 * required to override this to return the correct collection instance
+	 * based on the encoded child.
+	 */
+	protected Object getFieldTemplate(Object obj, String fieldname, Node child)
+	{
+		Object template = getFieldValue(obj, fieldname);
+
+		// Arrays are replaced completely
+		if (template != null && template.getClass().isArray())
+		{
+			template = null;
+		}
+		// Collections are cleared
+		else if (template instanceof Collection)
+		{
+			((Collection<?>) template).clear();
+		}
+
+		return template;
+	}
+
+	/**
+	 * Sets the decoded child node as a value of the given object. If the
+	 * object is a map, then the value is added with the given fieldname as a
+	 * key. If the fieldname is not empty, then setFieldValue is called or
+	 * else, if the object is a collection, the value is added to the
+	 * collection. For strongly typed languages it may be required to
+	 * override this with the correct code to add an entry to an object.
+	 */
+	protected void addObjectValue(Object obj, String fieldname, Object value,
+			Object template)
+	{
+		if (value != null && !value.equals(template))
+		{
+			if (fieldname != null && obj instanceof Map)
 			{
-				if (fieldname != null && obj instanceof Map)
-				{
-					((Map) obj).put(fieldname, value);
-				}
-				else if (fieldname != null && fieldname.length() > 0)
-				{
-					setFieldValue(obj, fieldname, value);
-				}
-				// Arrays are treated as collections and
-				// converted in setFieldValue
-				else if (obj instanceof Collection)
-				{
-					((Collection) obj).add(value);
-				}
+				((Map) obj).put(fieldname, value);
+			}
+			else if (fieldname != null && fieldname.length() > 0)
+			{
+				setFieldValue(obj, fieldname, value);
+			}
+			// Arrays are treated as collections and
+			// converted in setFieldValue
+			else if (obj instanceof Collection)
+			{
+				((Collection) obj).add(value);
 			}
 		}
 	}
