@@ -1,5 +1,5 @@
 /**
- * $Id: mxStylesheetCodec.java,v 1.10 2010-04-30 07:28:15 gaudenz Exp $
+ * $Id: mxStylesheetCodec.java,v 1.11 2011-06-13 08:18:42 gaudenz Exp $
  * Copyright (c) 2006-2010, JGraph Ltd
  */
 package com.mxgraph.io;
@@ -76,10 +76,9 @@ public class mxStylesheetCodec extends mxObjectCodec
 				{
 					Map.Entry<String, Object> entry2 = it2.next();
 					Element entryNode = enc.document.createElement("add");
-					entryNode.setAttribute("as", String
-							.valueOf(entry2.getKey()));
-					entryNode.setAttribute("value", String.valueOf(entry2
-							.getValue()));
+					entryNode.setAttribute("as",
+							String.valueOf(entry2.getKey()));
+					entryNode.setAttribute("value", getStringValue(entry2));
 					styleNode.appendChild(entryNode);
 				}
 
@@ -93,6 +92,19 @@ public class mxStylesheetCodec extends mxObjectCodec
 		return node;
 	}
 
+	/**
+	 * Returns the string for encoding the given value.
+	 */
+	protected String getStringValue(Map.Entry<String, Object> entry)
+	{
+		if (entry.getValue() instanceof Boolean)
+		{
+			return ((Boolean) entry.getValue()) ? "1" : "0";
+		}
+
+		return entry.getValue().toString();
+	}
+
 	/**
 	 * Decodes the given mxStylesheet.
 	 */
@@ -134,8 +146,7 @@ public class mxStylesheetCodec extends mxObjectCodec
 					{
 						String extend = ((Element) node).getAttribute("extend");
 						Map<String, Object> style = (extend != null) ? ((mxStylesheet) obj)
-								.getStyles().get(extend)
-								: null;
+								.getStyles().get(extend) : null;
 
 						if (style == null)
 						{
