@@ -5,6 +5,8 @@ import java.util.List;
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
 
+import com.mxgraph.util.mxUtils;
+
 /**
  * This class allows get the text contained in a shape formated with tags html.<br/>
  * The properties referenced in the Text element are processed by this class.
@@ -125,6 +127,8 @@ public class mxVdxTextParser
 				{
 					String text = e.getNodeValue();
 					text = textToList(text, pp);
+					// It's HTML text, fucking escape it.
+					text = mxUtils.htmlEntities(text);
 					text = text.replaceAll("\n", "<br/>");
 					ret += getTextCharFormated(text);
 
