@@ -1,5 +1,5 @@
 /*
- * $Id: mxGraphHierarchyModel.java,v 1.10 2010-08-02 13:26:46 david Exp $
+ * $Id: mxGraphHierarchyModel.java,v 1.12 2010-10-19 13:49:16 david Exp $
  * Copyright (c) 2005-2010, David Benson, Gaudenz Alder
  */
 package com.mxgraph.layout.hierarchical.model;
@@ -357,6 +357,7 @@ public class mxGraphHierarchyModel
 							if (layout.isDisableEdgeStyle())
 							{
 								layout.setEdgeStyleEnabled(edge, false);
+								layout.setOrthogonalEdge(edge,true);
 							}
 						}
 
@@ -929,7 +930,7 @@ public class mxGraphHierarchyModel
 	/**
 	 * @return Returns the vertexMapping.
 	 */
-	public Map<Object, mxGraphHierarchyNode> getVertexMapping()
+	public Map<Object, mxGraphHierarchyNode> getVertexMapper()
 	{
 		if (vertexMapper == null)
 		{
@@ -942,7 +943,7 @@ public class mxGraphHierarchyModel
 	 * @param vertexMapping
 	 *            The vertexMapping to set.
 	 */
-	public void setVertexMapping(Map<Object, mxGraphHierarchyNode> vertexMapping)
+	public void setVertexMapper(Map<Object, mxGraphHierarchyNode> vertexMapping)
 	{
 		this.vertexMapper = vertexMapping;
 	}
