@@ -1,6 +1,7 @@
 package com.mxgraph.layout;
 
 import java.util.HashSet;
+import java.util.List;
 import java.util.Set;
 
 import com.mxgraph.model.mxGeometry;
@@ -239,17 +240,17 @@ public class mxCompactTreeLayout extends mxGraphLayout
 			// children
 			else
 			{
-				Object[] roots = graph.findTreeRoots(parent, true, invert);
+				List<Object> roots = graph.findTreeRoots(parent, true, invert);
 
-				if (roots.length > 0)
+				if (roots.size() > 0)
 				{
-					for (int i = 0; i < roots.length; i++)
+					for (int i = 0; i < roots.size(); i++)
 					{
-						if (!isVertexIgnored(roots[i])
-								&& graph.getEdges(roots[i], null, invert,
+						if (!isVertexIgnored(roots.get(i))
+								&& graph.getEdges(roots.get(i), null, invert,
 										!invert, false).length > 0)
 						{
-							root = roots[i];
+							root = roots.get(i);
 							break;
 						}
 					}
