@@ -1,5 +1,5 @@
 /**
- * $Id: mxFastOrganicLayout.java,v 1.27 2010-02-11 11:11:16 david Exp $
+ * $Id: mxFastOrganicLayout.java,v 1.28 2011-04-28 13:10:32 david Exp $
  * Copyright (c) 2007, Gaudenz Alder
  */
 package com.mxgraph.layout;
@@ -59,6 +59,12 @@ public class mxFastOrganicLayout extends mxGraphLayout
 	 */
 	protected double minDistanceLimitSquared = 0;
 
+	/**
+	 * The maximum distance between vertices, beyond which their
+	 * repulsion no longer has an effect
+	 */
+	protected double maxDistanceLimit = 500;
+
 	/**
 	 * Start value of temperature. Default is 200.
 	 */
@@ -253,6 +259,22 @@ public class mxFastOrganicLayout extends mxGraphLayout
 		minDistanceLimit = value;
 	}
 
+	/**
+	 * @return the maxDistanceLimit
+	 */
+	public double getMaxDistanceLimit()
+	{
+		return maxDistanceLimit;
+	}
+
+	/**
+	 * @param maxDistanceLimit the maxDistanceLimit to set
+	 */
+	public void setMaxDistanceLimit(double maxDistanceLimit)
+	{
+		this.maxDistanceLimit = maxDistanceLimit;
+	}
+
 	/**
 	 * 
 	 */
@@ -625,6 +647,12 @@ public class mxFastOrganicLayout extends mxGraphLayout
 					double deltaLengthWithRadius = deltaLength - radius[i]
 							- radius[j];
 
+					if (deltaLengthWithRadius > maxDistanceLimit)
+					{
+						// Ignore vertices too far apart
+						continue;
+					}
+
 					if (deltaLengthWithRadius < minDistanceLimit)
 					{
 						deltaLengthWithRadius = minDistanceLimit;
