@@ -1,5 +1,5 @@
 /**
- * $Id: mxGraphLayout.java,v 1.20 2010-03-29 10:20:17 david Exp $
+ * $Id: mxGraphLayout.java,v 1.22 2010-10-19 13:49:16 david Exp $
  * Copyright (c) 2008-2009, JGraph Ltd
  */
 package com.mxgraph.layout;
@@ -155,6 +155,15 @@ public abstract class mxGraphLayout implements mxIGraphLayout
 		graph.setCellStyles(mxConstants.STYLE_NOEDGESTYLE, (value) ? "0" : "1",
 				new Object[] { edge });
 	}
+	
+	/**
+	 * Disables or enables orthogonal end segments of the given edge
+	 */
+	public void setOrthogonalEdge(Object edge, boolean value)
+	{
+		graph.setCellStyles(mxConstants.STYLE_ORTHOGONAL, (value) ? "1" : "0",
+				new Object[] { edge });
+	}
 
 	/**
 	 * Sets the control points of the given edge to the given
