@@ -1,5 +1,5 @@
 /**
- * $Id: mxOrganicLayout.java,v 1.6 2010-08-02 13:14:02 david Exp $
+ * $Id: mxOrganicLayout.java,v 1.9 2011-11-01 07:37:51 mate Exp $
  * Copyright (c) 2007-2009, JGraph Ltd
  */
 
@@ -153,7 +153,7 @@ public class mxOrganicLayout extends mxGraphLayout
 	 * is not set this value mutiplied by the number of nodes to find
 	 * the total graph area. The graph is assumed square.
 	 */
-	protected double averageNodeArea = 10000;
+	protected double averageNodeArea = 40000;
 
 	/**
 	 * The radius below which fine tuning of the layout should start
@@ -167,7 +167,7 @@ public class mxOrganicLayout extends mxGraphLayout
 	 * Limit to the number of iterations that may take place. This is only
 	 * reached if one of the termination conditions does not occur first.
 	 */
-	protected int maxIterations = 100;
+	protected int maxIterations = 1000;
 
 	/**
 	 * Cost factor applied to energy calculations involving the distance
@@ -176,7 +176,7 @@ public class mxOrganicLayout extends mxGraphLayout
 	 * <code>isOptimizeEdgeDistance</code> must be true for edge to nodes
 	 * distances to be taken into account.
 	 */
-	protected double edgeDistanceCostFactor = 4000;
+	protected double edgeDistanceCostFactor = 3000;
 
 	/**
 	 * Cost factor applied to energy calculations involving edges that cross
@@ -185,7 +185,7 @@ public class mxOrganicLayout extends mxGraphLayout
 	 * <code>isOptimizeEdgeCrossing</code> must be true for edge crossings
 	 * to be taken into account.
 	 */
-	protected double edgeCrossingCostFactor = 2000;
+	protected double edgeCrossingCostFactor = 6000;
 
 	/**
 	 * Cost factor applied to energy calculations involving the general node
@@ -195,7 +195,7 @@ public class mxOrganicLayout extends mxGraphLayout
 	 * <code>isOptimizeNodeDistribution</code> must be true for this general
 	 * distribution to be applied.
 	 */
-	protected double nodeDistributionCostFactor = 300000;
+	protected double nodeDistributionCostFactor = 30000;
 
 	/**
 	 * Cost factor applied to energy calculations for node promixity to the
@@ -332,6 +332,18 @@ public class mxOrganicLayout extends mxGraphLayout
 	 */
 	protected boolean isFineTuning = true;
 
+	/**
+	 *  Specifies if the STYLE_NOEDGESTYLE flag should be set on edges that are
+	 * modified by the result. Default is true.
+	 */
+	protected boolean disableEdgeStyle = true;
+
+	/**
+	 * Specifies if all edge points of traversed edges should be removed.
+	 * Default is true.
+	 */
+	protected boolean resetEdges = false;
+
 	/**
 	 * Constructor for mxOrganicLayout.
 	 */
@@ -373,29 +385,66 @@ public class mxOrganicLayout extends mxGraphLayout
 
 		// If the bounds dimensions have not been set see if the average area
 		// per node has been
-		mxRectangle bounds = graph.getBoundsForCells(vertices, false, false,
-				true);
+		mxRectangle totalBounds = null;
+		mxRectangle bounds = null;
+		
+		// Form internal model of nodes
+		Map<Object, Integer> vertexMap = new Hashtable<Object, Integer>();
+		v = new CellWrapper[vertices.length];
+		for (int i = 0; i < vertices.length; i++)
+		{
+			v[i] = new CellWrapper(vertices[i]);
+			vertexMap.put(vertices[i], new Integer(i));
+			bounds = getVertexBounds(vertices[i]);
+			
+			if (totalBounds == null)
+			{
+				totalBounds = (mxRectangle) bounds.clone();
+			}
+			else
+			{
+				totalBounds.add(bounds);
+			}
+
+			// Set the X,Y value of the internal version of the cell to
+			// the center point of the vertex for better positioning
+			double width = bounds.getWidth();
+			double height = bounds.getHeight();
+			v[i].x = bounds.getX() + width / 2.0;
+			v[i].y = bounds.getY() + height / 2.0;
+			if (approxNodeDimensions)
+			{
+				v[i].radiusSquared = Math.min(width, height);
+				v[i].radiusSquared *= v[i].radiusSquared;
+			}
+			else
+			{
+				v[i].radiusSquared = width * width;
+				v[i].heightSquared = height * height;
+			}
+		}
+
 		if (averageNodeArea == 0.0)
 		{
-			if (boundsWidth == 0.0 && bounds != null)
+			if (boundsWidth == 0.0 && totalBounds != null)
 			{
 				// Just use current bounds of graph
-				boundsX = bounds.getX();
-				boundsY = bounds.getY();
-				boundsWidth = bounds.getWidth();
-				boundsHeight = bounds.getHeight();
+				boundsX = totalBounds.getX();
+				boundsY = totalBounds.getY();
+				boundsWidth = totalBounds.getWidth();
+				boundsHeight = totalBounds.getHeight();
 			}
 		}
 		else
 		{
-			// find the centre point of the current graph
+			// find the center point of the current graph
 			// based the new graph bounds on the average node area set
 			double newArea = averageNodeArea * vertices.length;
 			double squareLength = Math.sqrt(newArea);
 			if (bounds != null)
 			{
-				double centreX = bounds.getX() + bounds.getWidth() / 2.0;
-				double centreY = bounds.getY() + bounds.getHeight() / 2.0;
+				double centreX = totalBounds.getX() + totalBounds.getWidth() / 2.0;
+				double centreY = totalBounds.getY() + totalBounds.getHeight() / 2.0;
 				boundsX = centreX - squareLength / 2.0;
 				boundsY = centreY - squareLength / 2.0;
 			}
@@ -430,31 +479,6 @@ public class mxOrganicLayout extends mxGraphLayout
 
 		unchangedEnergyRoundCount = 0;
 
-		// Form internal model of nodes
-		Map<Object, Integer> vertexMap = new Hashtable<Object, Integer>();
-		v = new CellWrapper[vertices.length];
-		for (int i = 0; i < vertices.length; i++)
-		{
-			v[i] = new CellWrapper(vertices[i]);
-			vertexMap.put(vertices[i], new Integer(i));
-			bounds = graph.getCellBounds(vertices[i]);
-			// Set the X,Y value of the internal version of the cell to
-			// the center point of the vertex for better positioning
-			double width = bounds.getWidth();
-			double height = bounds.getHeight();
-			v[i].x = bounds.getX() + width / 2.0;
-			v[i].y = bounds.getY() + height / 2.0;
-			if (approxNodeDimensions)
-			{
-				v[i].radiusSquared = Math.min(width, height);
-				v[i].radiusSquared *= v[i].radiusSquared;
-			}
-			else
-			{
-				v[i].radiusSquared = width * width;
-				v[i].heightSquared = height * height;
-			}
-		}
 
 		// Form internal model of edges
 		e = new CellWrapper[edges.length];
@@ -516,6 +540,27 @@ public class mxOrganicLayout extends mxGraphLayout
 			yNormTry[i] = Math.sin(angle);
 		}
 
+		
+		int childCount = model.getChildCount(parent);
+
+		for (int i = 0; i < childCount; i++)
+		{
+			Object cell = model.getChildAt(parent, i);
+
+			if (!isEdgeIgnored(cell))
+			{
+				if (isResetEdges())
+				{
+					graph.resetEdge(cell);
+				}
+
+				if (isDisableEdgeStyle())
+				{
+					setEdgeStyleEnabled(cell, false);
+				}
+			}
+		}
+		
 		// The main layout loop
 		for (iteration = 0; iteration < maxIterations; iteration++)
 		{
@@ -575,6 +620,8 @@ public class mxOrganicLayout extends mxGraphLayout
 			int index = i;
 
 			// Obtain the energies for the node is its current position
+			// TODO The energy could be stored from the last iteration
+			// and used again, rather than re-calculate
 			double oldNodeDistribution = getNodeDistribution(index);
 			double oldEdgeDistance = getEdgeDistanceFromNode(index);
 			oldEdgeDistance += getEdgeDistanceAffectedNodes(index);
@@ -1049,7 +1096,7 @@ public class mxOrganicLayout extends mxGraphLayout
 	protected double getEdgeDistanceAffectedNodes(int node)
 	{
 		double energy = 0.0;
-		for (int i = 0; i < v[node].connectedEdges.length; i++)
+		for (int i = 0; i < (v[node].connectedEdges.length); i++)
 		{
 			energy += getEdgeDistanceFromEdge(v[node].connectedEdges[i]);
 		}
@@ -1112,7 +1159,7 @@ public class mxOrganicLayout extends mxGraphLayout
 	}
 
 	/**
-	 * Returns all Edges that are not connected to the specifed cell
+	 * Returns all Edges that are not connected to the specified cell
 	 * 
 	 * @param cellIndex
 	 *            the cell index to which the edges are not connected
@@ -1745,4 +1792,36 @@ public class mxOrganicLayout extends mxGraphLayout
 	{
 		this.approxNodeDimensions = approxNodeDimensions;
 	}
+	
+	/**
+	 * @return the disableEdgeStyle
+	 */
+	public boolean isDisableEdgeStyle()
+	{
+		return disableEdgeStyle;
+	}
+
+	/**
+	 * @param disableEdgeStyle the disableEdgeStyle to set
+	 */
+	public void setDisableEdgeStyle(boolean disableEdgeStyle)
+	{
+		this.disableEdgeStyle = disableEdgeStyle;
+	}
+
+	/**
+	 * @return the resetEdges
+	 */
+	public boolean isResetEdges()
+	{
+		return resetEdges;
+	}
+
+	/**
+	 * @param resetEdges the resetEdges to set
+	 */
+	public void setResetEdges(boolean resetEdges)
+	{
+		this.resetEdges = resetEdges;
+	}
 }
