@@ -1,9 +1,12 @@
 /**
- * $Id: mxGraphModel.java,v 1.82 2010-08-02 13:49:12 david Exp $
+ * $Id: mxGraphModel.java,v 1.91 2011-03-21 16:52:13 gaudenz Exp $
  * Copyright (c) 2007, Gaudenz Alder
  */
 package com.mxgraph.model;
 
+import java.io.IOException;
+import java.io.ObjectInputStream;
+import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -42,9 +45,10 @@ import com.mxgraph.util.mxUndoableEdit;
  * 
  * This class fires the following events:
  * 
- * mxEvent.CHANGE fires when an undoable edit is dispatched. The
- * <code>changes</code> argument contains the list of undoable changes which is
- * stored in mxUndoableEdit.changes, the type is List&lt;mxUndoableChange&gt;.
+ * mxEvent.CHANGE fires when an undoable edit is dispatched. The <code>edit</code>
+ * property contains the mxUndoableEdit. The <code>changes</code> property
+ * contains the list of undoable changes inside the undoable edit. The changes
+ * property is deprecated, please use edit.getChanges() instead.
  * 
  * mxEvent.EXECUTE fires between begin- and endUpdate and after an atomic
  * change was executed in the model. The <code>change</code> property contains
@@ -64,7 +68,8 @@ import com.mxgraph.util.mxUndoableEdit;
  * mxEvent.UNDO fires after the change was dispatched in endUpdate. The
  * <code>edit</code> property contains the current mxUndoableEdit.
  */
-public class mxGraphModel extends mxEventSource implements mxIGraphModel
+public class mxGraphModel extends mxEventSource implements mxIGraphModel,
+		Serializable
 {
 
 	/**
@@ -286,8 +291,9 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 		{
 			public void dispatch()
 			{
+				// LATER: Remove changes property (deprecated)
 				((mxGraphModel) source).fireEvent(new mxEventObject(
-						mxEvent.CHANGE, "changes", changes));
+						mxEvent.CHANGE, "edit", this, "changes", changes));
 			}
 		};
 	}
@@ -725,7 +731,21 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 		Object source = getTerminal(edge, true);
 		Object target = getTerminal(edge, false);
 		Object cell = null;
-
+		
+		// Uses the first non-relative descendants of the source terminal
+		while (source != null && !isEdge(source) &&
+			getGeometry(source) != null && getGeometry(source).isRelative())
+		{
+			source = getParent(source);
+		}
+		
+		// Uses the first non-relative descendants of the target terminal
+		while (target != null && !isEdge(target) &&
+			getGeometry(target) != null && getGeometry(target).isRelative())
+		{
+			target = getParent(target);
+		}
+		
 		if (isAncestor(root, source) && isAncestor(root, target))
 		{
 			if (source == target)
@@ -955,7 +975,7 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 	 */
 	public String setStyle(Object cell, String style)
 	{
-		if (style != getStyle(cell))
+		if (style == null || !style.equals(getStyle(cell)))
 		{
 			execute(new mxStyleChange(this, cell, style));
 		}
@@ -1170,34 +1190,29 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 
 			for (int i = 0; i < childCount; i++)
 			{
-				Object child = from.getChildAt(i);
+				mxICell cell = from.getChildAt(i);
+				String id = cell.getId();
+				mxICell target = (mxICell) ((id != null && (!isEdge(cell) || !cloneAllEdges)) ? getCell(id)
+						: null);
 
-				if (child instanceof mxICell)
+				// Clones and adds the child if no cell exists for the id
+				if (target == null)
 				{
-					mxICell cell = (mxICell) child;
-					String id = cell.getId();
-					mxICell target = (mxICell) ((id != null && (!isEdge(cell) || !cloneAllEdges)) ? getCell(id)
-							: null);
+					mxCell clone = (mxCell) cell.clone();
+					clone.setId(id);
 
-					// Clones and adds the child if no cell exists for the id
-					if (target == null)
-					{
-						mxCell clone = (mxCell) cell.clone();
-						clone.setId(id);
-
-						// Do *NOT* use model.add as this will move the edge away
-						// from the parent in updateEdgeParent if maintainEdgeParent
-						// is enabled in the target model
-						target = to.insert(clone);
-						cellAdded(target);
-					}
-
-					// Stores the mapping for later reconnecting edges
-					mapping.put(cell, target);
-
-					// Recurses
-					mergeChildrenImpl(cell, target, cloneAllEdges, mapping);
+					// Do *NOT* use model.add as this will move the edge away
+					// from the parent in updateEdgeParent if maintainEdgeParent
+					// is enabled in the target model
+					target = to.insert(clone);
+					cellAdded(target);
 				}
+
+				// Stores the mapping for later reconnecting edges
+				mapping.put(cell, target);
+
+				// Recurses
+				mergeChildrenImpl(cell, target, cloneAllEdges, mapping);
 			}
 		}
 		finally
@@ -1206,6 +1221,16 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 		}
 	}
 
+	/**
+	 * Initializes the currentEdit field if the model is deserialized.
+	 */
+	private void readObject(ObjectInputStream ois) throws IOException,
+			ClassNotFoundException
+	{
+		ois.defaultReadObject();
+		currentEdit = createUndoableEdit();
+	}
+
 	/**
 	 * Returns the number of incoming or outgoing edges.
 	 * 
@@ -1710,6 +1735,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 		 */
 		protected Object root, previous;
 
+		/**
+		 * 
+		 */
+		public mxRootChange()
+		{
+			this(null, null);
+		}
+
 		/**
 		 * 
 		 */
@@ -1720,6 +1753,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			previous = root;
 		}
 
+		/**
+		 * 
+		 */
+		public void setRoot(Object value)
+		{
+			root = value;
+		}
+
 		/**
 		 * @return the root
 		 */
@@ -1728,6 +1769,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return root;
 		}
 
+		/**
+		 * 
+		 */
+		public void setPrevious(Object value)
+		{
+			previous = value;
+		}
+
 		/**
 		 * @return the previous
 		 */
@@ -1760,6 +1809,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 		 */
 		protected int index, previousIndex;
 
+		/**
+		 * 
+		 */
+		public mxChildChange()
+		{
+			this(null, null, null, 0);
+		}
+
 		/**
 		 * 
 		 */
@@ -1782,6 +1839,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			previousIndex = index;
 		}
 
+		/**
+		 *
+		 */
+		public void setParent(Object value)
+		{
+			parent = value;
+		}
+
 		/**
 		 * @return the parent
 		 */
@@ -1790,6 +1855,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return parent;
 		}
 
+		/**
+		 *
+		 */
+		public void setPrevious(Object value)
+		{
+			previous = value;
+		}
+
 		/**
 		 * @return the previous
 		 */
@@ -1798,6 +1871,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return previous;
 		}
 
+		/**
+		 *
+		 */
+		public void setChild(Object value)
+		{
+			child = value;
+		}
+
 		/**
 		 * @return the child
 		 */
@@ -1806,6 +1887,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return child;
 		}
 
+		/**
+		 *
+		 */
+		public void setIndex(int value)
+		{
+			index = value;
+		}
+
 		/**
 		 * @return the index
 		 */
@@ -1814,6 +1903,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return index;
 		}
 
+		/**
+		 *
+		 */
+		public void setPreviousIndex(int value)
+		{
+			previousIndex = value;
+		}
+
 		/**
 		 * @return the previousIndex
 		 */
@@ -1896,8 +1993,7 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 		protected int getChildIndex(Object parent, Object child)
 		{
 			return (parent instanceof mxICell && child instanceof mxICell) ? ((mxICell) parent)
-					.getIndex((mxICell) child)
-					: 0;
+					.getIndex((mxICell) child) : 0;
 		}
 
 		/**
@@ -1940,19 +2036,35 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 		/**
 		 * 
 		 */
-		protected boolean isSource;
+		protected boolean source;
+
+		/**
+		 * 
+		 */
+		public mxTerminalChange()
+		{
+			this(null, null, null, false);
+		}
 
 		/**
 		 * 
 		 */
 		public mxTerminalChange(mxGraphModel model, Object cell,
-				Object terminal, boolean isSource)
+				Object terminal, boolean source)
 		{
 			super(model);
 			this.cell = cell;
 			this.terminal = terminal;
 			this.previous = this.terminal;
-			this.isSource = isSource;
+			this.source = source;
+		}
+
+		/**
+		 * 
+		 */
+		public void setCell(Object value)
+		{
+			cell = value;
 		}
 
 		/**
@@ -1963,6 +2075,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return cell;
 		}
 
+		/**
+		 * 
+		 */
+		public void setTerminal(Object value)
+		{
+			terminal = value;
+		}
+
 		/**
 		 * @return the terminal
 		 */
@@ -1971,6 +2091,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return terminal;
 		}
 
+		/**
+		 * 
+		 */
+		public void setPrevious(Object value)
+		{
+			previous = value;
+		}
+
 		/**
 		 * @return the previous
 		 */
@@ -1979,12 +2107,20 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return previous;
 		}
 
+		/**
+		 * 
+		 */
+		public void setSource(boolean value)
+		{
+			source = value;
+		}
+
 		/**
 		 * @return the isSource
 		 */
 		public boolean isSource()
 		{
-			return isSource;
+			return source;
 		}
 
 		/**
@@ -1994,7 +2130,7 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 		{
 			terminal = previous;
 			previous = ((mxGraphModel) model).terminalForCellChanged(cell,
-					previous, isSource);
+					previous, source);
 		}
 
 	}
@@ -2007,6 +2143,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 		 */
 		protected Object cell, value, previous;
 
+		/**
+		 * 
+		 */
+		public mxValueChange()
+		{
+			this(null, null, null);
+		}
+
 		/**
 		 * 
 		 */
@@ -2018,6 +2162,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			this.previous = this.value;
 		}
 
+		/**
+		 * 
+		 */
+		public void setCell(Object value)
+		{
+			cell = value;
+		}
+
 		/**
 		 * @return the cell
 		 */
@@ -2026,6 +2178,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return cell;
 		}
 
+		/**
+		 * 
+		 */
+		public void setValue(Object value)
+		{
+			this.value = value;
+		}
+
 		/**
 		 * @return the value
 		 */
@@ -2034,6 +2194,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return value;
 		}
 
+		/**
+		 * 
+		 */
+		public void setPrevious(Object value)
+		{
+			previous = value;
+		}
+
 		/**
 		 * @return the previous
 		 */
@@ -2067,6 +2235,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 		 */
 		protected String style, previous;
 
+		/**
+		 * 
+		 */
+		public mxStyleChange()
+		{
+			this(null, null, null);
+		}
+
 		/**
 		 * 
 		 */
@@ -2078,6 +2254,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			this.previous = this.style;
 		}
 
+		/**
+		 * 
+		 */
+		public void setCell(Object value)
+		{
+			cell = value;
+		}
+
 		/**
 		 * @return the cell
 		 */
@@ -2086,6 +2270,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return cell;
 		}
 
+		/**
+		 * 
+		 */
+		public void setStyle(String value)
+		{
+			style = value;
+		}
+
 		/**
 		 * @return the style
 		 */
@@ -2094,6 +2286,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return style;
 		}
 
+		/**
+		 * 
+		 */
+		public void setPrevious(String value)
+		{
+			previous = value;
+		}
+
 		/**
 		 * @return the previous
 		 */
@@ -2127,6 +2327,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 		 */
 		protected mxGeometry geometry, previous;
 
+		/**
+		 * 
+		 */
+		public mxGeometryChange()
+		{
+			this(null, null, null);
+		}
+
 		/**
 		 * 
 		 */
@@ -2139,6 +2347,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			this.previous = this.geometry;
 		}
 
+		/**
+		 * 
+		 */
+		public void setCell(Object value)
+		{
+			cell = value;
+		}
+
 		/**
 		 * @return the cell
 		 */
@@ -2147,6 +2363,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return cell;
 		}
 
+		/**
+		 *
+		 */
+		public void setGeometry(mxGeometry value)
+		{
+			geometry = value;
+		}
+
 		/**
 		 * @return the geometry
 		 */
@@ -2155,6 +2379,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return geometry;
 		}
 
+		/**
+		 *
+		 */
+		public void setPrevious(mxGeometry value)
+		{
+			previous = value;
+		}
+
 		/**
 		 * @return the previous
 		 */
@@ -2188,6 +2420,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 		 */
 		protected boolean collapsed, previous;
 
+		/**
+		 * 
+		 */
+		public mxCollapseChange()
+		{
+			this(null, null, false);
+		}
+
 		/**
 		 * 
 		 */
@@ -2200,6 +2440,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			this.previous = this.collapsed;
 		}
 
+		/**
+		 * 
+		 */
+		public void setCell(Object value)
+		{
+			cell = value;
+		}
+
 		/**
 		 * @return the cell
 		 */
@@ -2208,6 +2456,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return cell;
 		}
 
+		/**
+		 * 
+		 */
+		public void setCollapsed(boolean value)
+		{
+			collapsed = value;
+		}
+
 		/**
 		 * @return the collapsed
 		 */
@@ -2216,6 +2472,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return collapsed;
 		}
 
+		/**
+		 * 
+		 */
+		public void setPrevious(boolean value)
+		{
+			previous = value;
+		}
+
 		/**
 		 * @return the previous
 		 */
@@ -2249,6 +2513,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 		 */
 		protected boolean visible, previous;
 
+		/**
+		 * 
+		 */
+		public mxVisibleChange()
+		{
+			this(null, null, false);
+		}
+
 		/**
 		 * 
 		 */
@@ -2260,6 +2532,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			this.previous = this.visible;
 		}
 
+		/**
+		 * 
+		 */
+		public void setCell(Object value)
+		{
+			cell = value;
+		}
+
 		/**
 		 * @return the cell
 		 */
@@ -2268,6 +2548,14 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return cell;
 		}
 
+		/**
+		 * 
+		 */
+		public void setVisible(boolean value)
+		{
+			visible = value;
+		}
+
 		/**
 		 * @return the visible
 		 */
@@ -2276,12 +2564,20 @@ public class mxGraphModel extends mxEventSource implements mxIGraphModel
 			return visible;
 		}
 
+		/**
+		 * 
+		 */
+		public void setPrevious(boolean value)
+		{
+			previous = value;
+		}
+
 		/**
 		 * @return the previous
 		 */
 		public boolean getPrevious()
 		{
-			return visible;
+			return previous;
 		}
 
 		/**
