@@ -1,5 +1,5 @@
 /**
- * $Id: mxStencilShape.java,v 1.8 2010-09-07 20:20:52 david Exp $
+ * $Id: mxStencilShape.java,v 1.11 2011-10-22 13:46:21 david Exp $
  * Copyright (c) 2010, Gaudenz Alder, David Benson
  */
 package com.mxgraph.shape;
@@ -29,16 +29,22 @@ import java.util.Map;
 
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
+import org.w3c.dom.NodeList;
 
 /**
- * Stencil shape drawing that takes an XML definition of the shape 
- * and renders it.
+ * Stencil shape drawing that takes an XML definition of the shape and renders
+ * it.
  * 
- * See http://projects.gnome.org/dia/custom-shapes for specs.
- * See http://dia-installer.de/shapes_de.html for shapes.
+ * See http://projects.gnome.org/dia/custom-shapes for specs. See
+ * http://dia-installer.de/shapes_de.html for shapes.
  */
 public class mxStencilShape extends mxBasicShape
 {
+	public mxStencilShape()
+	{
+		super();
+	}
+
 	protected GeneralPath shapePath;
 
 	/**
@@ -65,16 +71,49 @@ public class mxStencilShape extends mxBasicShape
 	 */
 	public mxStencilShape(String shapeXml)
 	{
-		Document document = mxUtils.parseXml(shapeXml);
+		this(mxUtils.parseXml(shapeXml));
+	}
 
-		this.name = document.getElementsByTagName("name").item(0)
-				.getTextContent();
-		this.iconPath = document.getElementsByTagName("icon").item(0)
-				.getTextContent();
-		this.root = document.getElementsByTagName("svg:svg").item(0);
+	public mxStencilShape(Document document)
+	{
+		if (document != null)
+		{
+			NodeList nameList = document.getElementsByTagName("name");
 
-		rootShape = new svgShape(null, null);
-		createShape(this.root, rootShape);
+			if (nameList != null && nameList.getLength() > 0)
+			{
+				this.name = nameList.item(0).getTextContent();
+			}
+
+			NodeList iconList = document.getElementsByTagName("icon");
+
+			if (iconList != null && iconList.getLength() > 0)
+			{
+				this.iconPath = iconList.item(0).getTextContent();
+			}
+
+			NodeList svgList = document.getElementsByTagName("svg:svg");
+
+			if (svgList != null && svgList.getLength() > 0)
+			{
+				this.root = svgList.item(0);
+			}
+			else
+			{
+				svgList = document.getElementsByTagName("svg");
+
+				if (svgList != null && svgList.getLength() > 0)
+				{
+					this.root = svgList.item(0);
+				}
+			}
+
+			if (this.root != null)
+			{
+				rootShape = new svgShape(null, null);
+				createShape(this.root, rootShape);
+			}
+		}
 	}
 
 	/**
@@ -135,10 +174,11 @@ public class mxStencilShape extends mxBasicShape
 				else if (strokeStyle.trim().startsWith("#"))
 				{
 					int hashIndex = strokeStyle.indexOf("#");
-					strokeColor = mxUtils.parseColor(strokeStyle.substring(hashIndex + 1));
+					strokeColor = mxUtils.parseColor(strokeStyle
+							.substring(hashIndex + 1));
 				}
 			}
-			
+
 			if (fillStyle != null)
 			{
 				if (fillStyle.equals(CSSConstants.CSS_NONE_VALUE))
@@ -148,7 +188,8 @@ public class mxStencilShape extends mxBasicShape
 				else if (fillStyle.trim().startsWith("#"))
 				{
 					int hashIndex = fillStyle.indexOf("#");
-					fillColor = mxUtils.parseColor(fillStyle.substring(hashIndex + 1));
+					fillColor = mxUtils.parseColor(fillStyle
+							.substring(hashIndex + 1));
 					fill = true;
 				}
 				else
@@ -176,7 +217,7 @@ public class mxStencilShape extends mxBasicShape
 				{
 					canvas.getGraphics().setColor(fillColor);
 				}
-				
+
 				canvas.getGraphics().fill(associatedShape);
 			}
 
@@ -187,7 +228,7 @@ public class mxStencilShape extends mxBasicShape
 				{
 					canvas.getGraphics().setColor(strokeColor);
 				}
-				
+
 				canvas.getGraphics().draw(associatedShape);
 			}
 
@@ -199,7 +240,8 @@ public class mxStencilShape extends mxBasicShape
 		}
 
 		/*
-		 * If root is a group element, then we should add it's styles to the children.
+		 * If root is a group element, then we should add it's styles to the
+		 * children.
 		 */
 		for (svgShape subShape : shape.subShapes)
 		{
@@ -209,11 +251,17 @@ public class mxStencilShape extends mxBasicShape
 
 	/**
 	 * Scales the points composing this shape by the x and y ratios specified
-	 * @param shape the shape to scale
-	 * @param transX the x translation
-	 * @param transY the y translation
-	 * @param widthRatio the x co-ordinate scale
-	 * @param heightRatio the y co-ordinate scale
+	 * 
+	 * @param shape
+	 *            the shape to scale
+	 * @param transX
+	 *            the x translation
+	 * @param transY
+	 *            the y translation
+	 * @param widthRatio
+	 *            the x co-ordinate scale
+	 * @param heightRatio
+	 *            the y co-ordinate scale
 	 */
 	protected void transformShape(Shape shape, double transX, double transY,
 			double widthRatio, double heightRatio)
@@ -223,16 +271,15 @@ public class mxStencilShape extends mxBasicShape
 			Rectangle2D rect = (Rectangle2D) shape;
 			if (transX != 0 || transY != 0)
 			{
-				rect.setFrame(rect.getX() + transX, rect.getY() + transY, rect
-						.getWidth(), rect.getHeight());
+				rect.setFrame(rect.getX() + transX, rect.getY() + transY,
+						rect.getWidth(), rect.getHeight());
 			}
 
 			if (widthRatio != 1 || heightRatio != 1)
 			{
 				rect.setFrame(rect.getX() * widthRatio, rect.getY()
-						* heightRatio, rect.getWidth() * widthRatio, rect
-						.getHeight()
-						* heightRatio);
+						* heightRatio, rect.getWidth() * widthRatio,
+						rect.getHeight() * heightRatio);
 			}
 		}
 		else if (shape instanceof Line2D)
@@ -240,9 +287,8 @@ public class mxStencilShape extends mxBasicShape
 			Line2D line = (Line2D) shape;
 			if (transX != 0 || transY != 0)
 			{
-				line.setLine(line.getX1() + transX, line.getY1() + transY, line
-						.getX2()
-						+ transX, line.getY2() + transY);
+				line.setLine(line.getX1() + transX, line.getY1() + transY,
+						line.getX2() + transX, line.getY2() + transY);
 			}
 			if (widthRatio != 1 || heightRatio != 1)
 			{
@@ -276,9 +322,8 @@ public class mxStencilShape extends mxBasicShape
 			if (widthRatio != 1 || heightRatio != 1)
 			{
 				ellipse.setFrame(ellipse.getX() * widthRatio, ellipse.getY()
-						* heightRatio, ellipse.getWidth() * widthRatio, ellipse
-						.getHeight()
-						* heightRatio);
+						* heightRatio, ellipse.getWidth() * widthRatio,
+						ellipse.getHeight() * heightRatio);
 			}
 		}
 	}
@@ -290,10 +335,20 @@ public class mxStencilShape extends mxBasicShape
 	{
 		Node child = root.getFirstChild();
 		/*
-		 * If root is a group element, then we should add it's styles to the childrens...
+		 * If root is a group element, then we should add it's styles to the
+		 * childrens...
 		 */
 		while (child != null)
 		{
+			if (isGroup(child.getNodeName()))
+			{
+				String style = ((Element) root).getAttribute("style");
+				Map<String, Object> styleMap = mxStencilShape
+						.getStylenames(style);
+				svgShape subShape = new svgShape(null, styleMap);
+				createShape(child, subShape);
+			}
+
 			svgShape subShape = createElement(child);
 
 			if (subShape != null)
@@ -320,8 +375,8 @@ public class mxStencilShape extends mxBasicShape
 
 		// If the shape does not butt up against either or both axis,
 		// ensure it is flush against both
-		if (boundingBox != null && boundingBox.getX() != 0
-				|| boundingBox.getY() != 0)
+		if (boundingBox != null
+				&& (boundingBox.getX() != 0 || boundingBox.getY() != 0))
 		{
 			for (svgShape subShape : shape.subShapes)
 			{
@@ -335,10 +390,13 @@ public class mxStencilShape extends mxBasicShape
 	}
 
 	/**
-	 * Forms an internal representation of the specified SVG element and 
-	 * returns that representation
-	 * @param root the SVG element to represent
-	 * @return the internal representation of the element, or null if an error occurs
+	 * Forms an internal representation of the specified SVG element and returns
+	 * that representation
+	 * 
+	 * @param root
+	 *            the SVG element to represent
+	 * @return the internal representation of the element, or null if an error
+	 *         occurs
 	 */
 	public svgShape createElement(Node root)
 	{
@@ -597,7 +655,7 @@ public class mxStencilShape extends mxBasicShape
 	 */
 	private boolean isRectangle(String tag)
 	{
-		return tag.equals("svg:rect");
+		return tag.equals("svg:rect") || tag.equals("rect");
 	}
 
 	/*
@@ -605,7 +663,7 @@ public class mxStencilShape extends mxBasicShape
 	 */
 	private boolean isPath(String tag)
 	{
-		return tag.equals("svg:path");
+		return tag.equals("svg:path") || tag.equals("path");
 	}
 
 	/*
@@ -613,7 +671,7 @@ public class mxStencilShape extends mxBasicShape
 	 */
 	private boolean isEllipse(String tag)
 	{
-		return tag.equals("svg:ellipse");
+		return tag.equals("svg:ellipse") || tag.equals("ellipse");
 	}
 
 	/*
@@ -621,7 +679,7 @@ public class mxStencilShape extends mxBasicShape
 	 */
 	private boolean isLine(String tag)
 	{
-		return tag.equals("svg:line");
+		return tag.equals("svg:line") || tag.equals("line");
 	}
 
 	/*
@@ -629,7 +687,7 @@ public class mxStencilShape extends mxBasicShape
 	 */
 	private boolean isPolyline(String tag)
 	{
-		return tag.equals("svg:polyline");
+		return tag.equals("svg:polyline") || tag.equals("polyline");
 	}
 
 	/*
@@ -637,7 +695,7 @@ public class mxStencilShape extends mxBasicShape
 	 */
 	private boolean isCircle(String tag)
 	{
-		return tag.equals("svg:circle");
+		return tag.equals("svg:circle") || tag.equals("circle");
 	}
 
 	/*
@@ -645,12 +703,17 @@ public class mxStencilShape extends mxBasicShape
 	 */
 	private boolean isPolygon(String tag)
 	{
-		return tag.equals("svg:polygon");
+		return tag.equals("svg:polygon") || tag.equals("polygon");
+	}
+
+	private boolean isGroup(String tag)
+	{
+		return tag.equals("svg:g") || tag.equals("g");
 	}
 
 	protected class svgShape
 	{
-		protected Shape shape;
+		public Shape shape;
 
 		/**
 		 * Contains an array of key, value pairs that represent the style of the
@@ -658,7 +721,7 @@ public class mxStencilShape extends mxBasicShape
 		 */
 		protected Map<String, Object> style;
 
-		protected List<svgShape> subShapes;
+		public List<svgShape> subShapes;
 
 		/**
 		 * Holds the current value to which the shape is scaled in X
@@ -751,4 +814,14 @@ public class mxStencilShape extends mxBasicShape
 	{
 		this.iconPath = iconPath;
 	}
+
+	public Rectangle2D getBoundingBox()
+	{
+		return boundingBox;
+	}
+
+	public void setBoundingBox(Rectangle2D boundingBox)
+	{
+		this.boundingBox = boundingBox;
+	}
 }
\ No newline at end of file
