@@ -1,5 +1,5 @@
 /**
- * $Id: mxCellHandler.java,v 1.24 2010-06-25 13:16:42 gaudenz Exp $
+ * $Id: mxCellHandler.java,v 1.25 2011-01-17 11:14:44 gaudenz Exp $
  * Copyright (c) 2008, Gaudenz Alder
  */
 package com.mxgraph.swing.handler;
@@ -9,6 +9,7 @@ import java.awt.Cursor;
 import java.awt.Graphics;
 import java.awt.Point;
 import java.awt.Rectangle;
+import java.awt.Stroke;
 import java.awt.event.MouseEvent;
 
 import javax.swing.JComponent;
@@ -371,6 +372,24 @@ public class mxCellHandler
 		}
 	}
 
+	/**
+	 * Returns the color used to draw the selection border. This implementation
+	 * returns null.
+	 */
+	public Color getSelectionColor()
+	{
+		return null;
+	}
+
+	/**
+	 * Returns the stroke used to draw the selection border. This implementation
+	 * returns null.
+	 */
+	public Stroke getSelectionStroke()
+	{
+		return null;
+	}
+
 	/**
 	 * Returns true if the handle at the specified index is enabled.
 	 */
