@@ -1,5 +1,5 @@
 /**
- * $Id: mxCellMarker.java,v 1.19 2010-07-21 10:59:02 gaudenz Exp $
+ * $Id: mxCellMarker.java,v 1.21 2011-09-30 06:01:58 gaudenz Exp $
  * Copyright (c) 2008, Gaudenz Alder
  */
 package com.mxgraph.swing.handler;
@@ -20,8 +20,8 @@ import com.mxgraph.util.mxConstants;
 import com.mxgraph.util.mxEvent;
 import com.mxgraph.util.mxEventObject;
 import com.mxgraph.util.mxEventSource;
-import com.mxgraph.util.mxUtils;
 import com.mxgraph.util.mxEventSource.mxIEventListener;
+import com.mxgraph.util.mxUtils;
 import com.mxgraph.view.mxCellState;
 import com.mxgraph.view.mxGraphView;
 
@@ -33,6 +33,87 @@ import com.mxgraph.view.mxGraphView;
  * mxEvent.MARK fires in mark and unmark to notify the listener of a new cell
  * under the mouse. The <code>state</code> property contains the mxCellState
  * of the respective cell or null if no cell is under the mouse.
+ * 
+ * To create a cell marker which highlights cells "in-place", the following
+ * code can be used:
+ * <code>
+ * mxCellMarker highlighter = new mxCellMarker(graphComponent) {
+ * 
+ *   protected Map<String, Object> lastStyle;
+ *   
+ *   public mxCellState process(MouseEvent e)
+ *   {
+ *     mxCellState state = null;
+ *     
+ *     if (isEnabled())
+ *     {
+ *       state = getState(e);
+ *       boolean isValid = (state != null) ? isValidState(state) : false;
+ *       
+ *       if (!isValid)
+ *       {
+ *         state = null;
+ *       }
+ *       
+ *       highlight(state);
+ *     }
+ *     
+ *     return state;
+ *   }
+ *   
+ *   public void highlight(mxCellState state)
+ *   {
+ *     if (validState != state)
+ *     {
+ *       Rectangle dirty = null;
+ *       
+ *       if (validState != null)
+ *       {
+ *         validState.setStyle(lastStyle);
+ *         dirty = validState.getBoundingBox().getRectangle();
+ *         dirty.grow(4, 4);
+ *       }
+ *       
+ *       if (state != null)
+ *       {
+ *         lastStyle = state.getStyle();
+ *         state.setStyle(new Hashtable<String, Object>(state.getStyle()));
+ *         state.getStyle().put("strokeColor", "#00ff00");
+ *         state.getStyle().put("fontColor", "#00ff00");
+ *         state.getStyle().put("strokeWidth", "3");
+ *          
+ *         Rectangle tmp = state.getBoundingBox().getRectangle();
+ *         
+ *         if (dirty != null)
+ *         {
+ *           dirty.add(tmp);
+ *         }
+ *         else
+ *         {
+ *           dirty = tmp;
+ *         }
+ *         
+ *         dirty.grow(4, 4);
+ *       }
+ *       
+ *       validState = state;
+ *       graphComponent.repaint(dirty);
+ *     }
+ *   }
+ *
+ *   public void reset()
+ *   {
+ *     highlight(null);
+ *   }
+ *
+ *   public void paint(Graphics g)
+ *   {
+ *     // do nothing
+ *   }
+ * };
+ *  
+ * graphComponent.getConnectionHandler().setMarker(highlighter);
+ * </code>
  */
 public class mxCellMarker extends JComponent
 {
@@ -323,37 +404,53 @@ public class mxCellMarker extends JComponent
 		if (isEnabled())
 		{
 			state = getState(e);
-			boolean isValid = (state != null) ? isValidState(state) : false;
-			Color color = getMarkerColor(e, state, isValid);
-
-			if (isValid)
-			{
-				validState = state;
-			}
-			else
-			{
-				validState = null;
-			}
-
-			if (state != markedState || color != currentColor)
-			{
-				currentColor = color;
-
-				if (state != null && currentColor != null)
-				{
-					markedState = state;
-					mark();
-				}
-				else if (markedState != null)
-				{
-					markedState = null;
-					unmark();
-				}
-			}
+			boolean valid = (state != null) ? isValidState(state) : false;
+			Color color = getMarkerColor(e, state, valid);
+			
+			highlight(state, color, valid);
 		}
 
 		return state;
 	}
+	
+	/**
+	 * 
+	 */
+	public void highlight(mxCellState state, Color color)
+	{
+		highlight(state, color, true);
+	}
+	
+	/**
+	 * 
+	 */
+	public void highlight(mxCellState state, Color color, boolean valid)
+	{
+		if (valid)
+		{
+			validState = state;
+		}
+		else
+		{
+			validState = null;
+		}
+
+		if (state != markedState || color != currentColor)
+		{
+			currentColor = color;
+
+			if (state != null && currentColor != null)
+			{
+				markedState = state;
+				mark();
+			}
+			else if (markedState != null)
+			{
+				markedState = null;
+				unmark();
+			}
+		}
+	}
 
 	/**
 	 * Marks the markedState and fires a mxEvent.MARK event.
