@@ -1,5 +1,5 @@
 /**
- * $Id: mxConnectionHandler.java,v 1.25 2010-08-23 15:27:16 gaudenz Exp $
+ * $Id: mxConnectionHandler.java,v 1.30 2011-09-05 08:43:32 gaudenz Exp $
  * Copyright (c) 2008, Gaudenz Alder
  */
 package com.mxgraph.swing.handler;
@@ -10,6 +10,8 @@ import java.awt.Graphics;
 import java.awt.Point;
 import java.awt.Rectangle;
 import java.awt.event.MouseEvent;
+import java.beans.PropertyChangeEvent;
+import java.beans.PropertyChangeListener;
 
 import javax.swing.ImageIcon;
 import javax.swing.JOptionPane;
@@ -22,9 +24,10 @@ import com.mxgraph.swing.util.mxMouseAdapter;
 import com.mxgraph.util.mxConstants;
 import com.mxgraph.util.mxEvent;
 import com.mxgraph.util.mxEventObject;
+import com.mxgraph.util.mxEventSource;
+import com.mxgraph.util.mxEventSource.mxIEventListener;
 import com.mxgraph.util.mxPoint;
 import com.mxgraph.util.mxRectangle;
-import com.mxgraph.util.mxEventSource.mxIEventListener;
 import com.mxgraph.view.mxCellState;
 import com.mxgraph.view.mxGraph;
 import com.mxgraph.view.mxGraphView;
@@ -32,6 +35,10 @@ import com.mxgraph.view.mxGraphView;
 /**
  * Connection handler creates new connections between cells. This control is used to display the connector
  * icon, while the preview is used to draw the line.
+ * 
+ * mxEvent.CONNECT fires between begin- and endUpdate in mouseReleased. The <code>cell</code>
+ * property contains the inserted edge, the <code>event</code> and <code>target</code> 
+ * properties contain the respective arguments that were passed to mouseReleased.
  */
 public class mxConnectionHandler extends mxMouseAdapter
 {
@@ -51,6 +58,11 @@ public class mxConnectionHandler extends mxMouseAdapter
 	 */
 	protected mxGraphComponent graphComponent;
 
+	/**
+	 * Holds the event source.
+	 */
+	protected mxEventSource eventSource = new mxEventSource(this);
+
 	/**
 	 * 
 	 */
@@ -92,6 +104,11 @@ public class mxConnectionHandler extends mxMouseAdapter
 	 */
 	protected boolean keepOnTop = true;
 
+	/**
+	 * 
+	 */
+	protected boolean enabled = true;
+
 	/**
 	 * 
 	 */
@@ -117,11 +134,6 @@ public class mxConnectionHandler extends mxMouseAdapter
 	 */
 	protected transient String error;
 
-	/**
-	 * 
-	 */
-	protected boolean enabled = true;
-
 	/**
 	 * 
 	 */
@@ -157,13 +169,20 @@ public class mxConnectionHandler extends mxMouseAdapter
 		graphControl.addMouseListener(this);
 		graphControl.addMouseMotionListener(this);
 
-		mxGraphView view = graphComponent.getGraph().getView();
-		view.addListener(mxEvent.SCALE, resetHandler);
-		view.addListener(mxEvent.TRANSLATE, resetHandler);
-		view.addListener(mxEvent.SCALE_AND_TRANSLATE, resetHandler);
+		// Installs the graph listeners and keeps them in sync
+		addGraphListeners(graphComponent.getGraph());
 
-		mxIGraphModel model = graphComponent.getGraph().getModel();
-		model.addListener(mxEvent.CHANGE, resetHandler);
+		graphComponent.addPropertyChangeListener(new PropertyChangeListener()
+		{
+			public void propertyChange(PropertyChangeEvent evt)
+			{
+				if (evt.getPropertyName().equals("graph"))
+				{
+					removeGraphListeners((mxGraph) evt.getOldValue());
+					addGraphListeners((mxGraph) evt.getNewValue());
+				}
+			}
+		});
 
 		marker = new mxCellMarker(graphComponent)
 		{
@@ -243,6 +262,39 @@ public class mxConnectionHandler extends mxMouseAdapter
 		marker.setHotspotEnabled(true);
 	}
 
+	/**
+	 * Installs the listeners to update the handles after any changes.
+	 */
+	protected void addGraphListeners(mxGraph graph)
+	{
+		// LATER: Install change listener for graph model, view
+		if (graph != null)
+		{
+			mxGraphView view = graph.getView();
+			view.addListener(mxEvent.SCALE, resetHandler);
+			view.addListener(mxEvent.TRANSLATE, resetHandler);
+			view.addListener(mxEvent.SCALE_AND_TRANSLATE, resetHandler);
+
+			graph.getModel().addListener(mxEvent.CHANGE, resetHandler);
+		}
+	}
+
+	/**
+	 * Removes all installed listeners.
+	 */
+	protected void removeGraphListeners(mxGraph graph)
+	{
+		if (graph != null)
+		{
+			mxGraphView view = graph.getView();
+			view.removeListener(resetHandler, mxEvent.SCALE);
+			view.removeListener(resetHandler, mxEvent.TRANSLATE);
+			view.removeListener(resetHandler, mxEvent.SCALE_AND_TRANSLATE);
+
+			graph.getModel().removeListener(resetHandler, mxEvent.CHANGE);
+		}
+	}
+
 	/**
 	 * 
 	 */
@@ -258,7 +310,7 @@ public class mxConnectionHandler extends mxMouseAdapter
 	{
 		return connectPreview;
 	}
-	
+
 	/**
 	 * 
 	 */
@@ -578,11 +630,8 @@ public class mxConnectionHandler extends mxMouseAdapter
 
 			if (connectPreview.isActive())
 			{
-				mxGraph graph = graphComponent.getGraph();
-				double x = graph.snap(e.getX());
-				double y = graph.snap(e.getY());
-
-				connectPreview.update(e, marker.getValidState(), x, y);
+				connectPreview.update(e, marker.getValidState(), e.getX(),
+						e.getY());
 				setBounds(null);
 				e.consume();
 			}
@@ -619,10 +668,12 @@ public class mxConnectionHandler extends mxMouseAdapter
 
 				try
 				{
+					Object dropTarget = null;
+
 					if (!marker.hasValidState() && isCreateTarget())
 					{
 						Object vertex = createTargetVertex(e, source.getCell());
-						Object dropTarget = graph.getDropTarget(
+						dropTarget = graph.getDropTarget(
 								new Object[] { vertex }, e.getPoint(),
 								graphComponent.getCellAt(e.getX(), e.getY()));
 
@@ -653,24 +704,25 @@ public class mxConnectionHandler extends mxMouseAdapter
 							graph.addCells(new Object[] { vertex }, dropTarget);
 						}
 
-						double x = graph.snap(e.getX());
-						double y = graph.snap(e.getY());
-
 						// FIXME: Here we pre-create the state for the vertex to be
 						// inserted in order to invoke update in the connectPreview.
 						// This means we have a cell state which should be created
 						// after the model.update, so this should be fixed.
 						mxCellState targetState = graph.getView().getState(
 								vertex, true);
-						connectPreview.update(e, targetState, x, y);
+						connectPreview.update(e, targetState, e.getX(),
+								e.getY());
 					}
 
-					Object cell = connectPreview.stop(graphComponent
-							.isSignificant(dx, dy), e);
+					Object cell = connectPreview.stop(
+							graphComponent.isSignificant(dx, dy), e);
 
 					if (cell != null)
 					{
 						graphComponent.getGraph().setSelectionCell(cell);
+						eventSource.fireEvent(new mxEventObject(
+								mxEvent.CONNECT, "cell", cell, "event", e,
+								"target", dropTarget));
 					}
 
 					e.consume();
@@ -717,6 +769,30 @@ public class mxConnectionHandler extends mxMouseAdapter
 		}
 	}
 
+	/**
+	 * Adds the given event listener.
+	 */
+	public void addListener(String eventName, mxIEventListener listener)
+	{
+		eventSource.addListener(eventName, listener);
+	}
+
+	/**
+	 * Removes the given event listener.
+	 */
+	public void removeListener(mxIEventListener listener)
+	{
+		eventSource.removeListener(listener);
+	}
+
+	/**
+	 * Removes the given event listener for the specified event name.
+	 */
+	public void removeListener(mxIEventListener listener, String eventName)
+	{
+		eventSource.removeListener(listener, eventName);
+	}
+
 	/**
 	 * 
 	 */
@@ -743,4 +819,5 @@ public class mxConnectionHandler extends mxMouseAdapter
 			}
 		}
 	}
+
 }
