@@ -1,5 +1,5 @@
 /**
- * $Id: mxEdgeHandler.java,v 1.25 2010-06-01 10:18:45 gaudenz Exp $
+ * $Id: mxEdgeHandler.java,v 1.29 2011-10-20 17:00:14 gaudenz Exp $
  * Copyright (c) 2008, Gaudenz Alder
  */
 package com.mxgraph.swing.handler;
@@ -92,11 +92,15 @@ public class mxEdgeHandler extends mxCellHandler
 		// Sets the highlight color according to isValidConnection
 		protected boolean isValidState(mxCellState state)
 		{
+			mxGraphView view = graphComponent.getGraph().getView();
 			mxIGraphModel model = graphComponent.getGraph().getModel();
 			Object edge = mxEdgeHandler.this.state.getCell();
 			boolean isSource = isSource(index);
 
-			Object other = model.getTerminal(edge, !isSource);
+			mxCellState other = view
+					.getTerminalPort(state,
+							view.getState(model.getTerminal(edge, !isSource)),
+							isSource);
 			Object source = (isSource) ? state.getCell() : other;
 			Object target = (isSource) ? other : state.getCell();
 
@@ -223,10 +227,16 @@ public class mxEdgeHandler extends mxCellHandler
 			Object terminal = graph.getModel().getTerminal(state.getCell(),
 					source);
 
-			if (terminal != null)
+			if (terminal == null
+					&& !graphComponent.getGraph().isTerminalPointMovable(
+							state.getCell(), source))
 			{
-				return (graphComponent.getGraph().isCellDisconnectable(state
-						.getCell(), terminal, source)) ? mxConstants.CONNECT_HANDLE_FILLCOLOR
+				return mxConstants.LOCKED_HANDLE_FILLCOLOR;
+			}
+			else if (terminal != null)
+			{
+				return (graphComponent.getGraph().isCellDisconnectable(
+						state.getCell(), terminal, source)) ? mxConstants.CONNECT_HANDLE_FILLCOLOR
 						: mxConstants.LOCKED_HANDLE_FILLCOLOR;
 			}
 		}
@@ -409,15 +419,18 @@ public class mxEdgeHandler extends mxCellHandler
 	{
 		super.mousePressed(e);
 
-		if (isSource(index) || isTarget(index))
+		boolean source = isSource(index);
+
+		if (source || isTarget(index))
 		{
 			mxGraph graph = graphComponent.getGraph();
 			mxIGraphModel model = graph.getModel();
-			Object terminal = model.getTerminal(state.getCell(),
-					isSource(index));
+			Object terminal = model.getTerminal(state.getCell(), source);
 
-			if (!graph.isCellDisconnectable(state.getCell(), terminal,
-					isSource(index)))
+			if ((terminal == null && !graph.isTerminalPointMovable(
+					state.getCell(), source))
+					|| (terminal != null && !graph.isCellDisconnectable(
+							state.getCell(), terminal, source)))
 			{
 				first = null;
 			}
@@ -583,9 +596,8 @@ public class mxEdgeHandler extends mxCellHandler
 			}
 
 			if (!preview.isVisible()
-					&& graphComponent.isSignificant(e.getX() - first.x, e
-							.getY()
-							- first.y))
+					&& graphComponent.isSignificant(e.getX() - first.x,
+							e.getY() - first.y))
 			{
 				preview.setVisible(true);
 			}
@@ -657,8 +669,11 @@ public class mxEdgeHandler extends mxCellHandler
 				else if ((!isSource(index) && !isTarget(index))
 						|| graphComponent.getGraph().isAllowDanglingEdges())
 				{
-					movePoint(state.getCell(), index, convertPoint(new mxPoint(
-							e.getPoint()), gridEnabledEvent));
+					movePoint(
+							state.getCell(),
+							index,
+							convertPoint(new mxPoint(e.getPoint()),
+									gridEnabledEvent));
 				}
 
 				e.consume();
@@ -829,7 +844,7 @@ public class mxEdgeHandler extends mxCellHandler
 	/**
 	 * 
 	 */
-	protected Color getSelectionColor()
+	public Color getSelectionColor()
 	{
 		return mxConstants.EDGE_SELECTION_COLOR;
 	}
@@ -837,7 +852,7 @@ public class mxEdgeHandler extends mxCellHandler
 	/**
 	 * 
 	 */
-	protected Stroke getSelectionStroke()
+	public Stroke getSelectionStroke()
 	{
 		return mxConstants.EDGE_SELECTION_STROKE;
 	}
@@ -859,14 +874,15 @@ public class mxEdgeHandler extends mxCellHandler
 		{
 			Point current = state.getAbsolutePoint(i).getPoint();
 			Line2D line = new Line2D.Float(last.x, last.y, current.x, current.y);
-			
-			Rectangle bounds = g2.getStroke().createStrokedShape(line).getBounds();
+
+			Rectangle bounds = g2.getStroke().createStrokedShape(line)
+					.getBounds();
 
 			if (g.hitClip(bounds.x, bounds.y, bounds.width, bounds.height))
 			{
 				g2.draw(line);
 			}
-			
+
 			last = current;
 		}
 
