@@ -1,5 +1,5 @@
 /**
- * $Id: mxGraphHandler.java,v 1.70 2010-08-23 15:19:23 gaudenz Exp $
+ * $Id: mxGraphHandler.java,v 1.74 2011-11-04 12:46:37 gaudenz Exp $
  * Copyright (c) 2008, Gaudenz Alder
  * 
  * Known issue: Drag image size depends on the initial position and may sometimes
@@ -31,6 +31,8 @@ import java.awt.dnd.DropTargetEvent;
 import java.awt.dnd.DropTargetListener;
 import java.awt.event.InputEvent;
 import java.awt.event.MouseEvent;
+import java.beans.PropertyChangeEvent;
+import java.beans.PropertyChangeListener;
 import java.util.TooManyListenersException;
 
 import javax.swing.ImageIcon;
@@ -215,6 +217,11 @@ public class mxGraphHandler extends mxMouseAdapter implements
 	 */
 	protected transient boolean constrainedEvent = false;
 
+	/**
+	 * Reference to the current drop target.
+	 */
+	protected transient DropTarget currentDropTarget = null;
+
 	/**
 	 * 
 	 * @param graphComponent
@@ -244,6 +251,24 @@ public class mxGraphHandler extends mxMouseAdapter implements
 
 		// Listens to dropped graph cells
 		installDropTargetHandler();
+		
+		// Listens to changes of the transferhandler
+		graphComponent.addPropertyChangeListener(new PropertyChangeListener()
+		{
+			
+			public void propertyChange(PropertyChangeEvent evt)
+			{
+				if (evt.getPropertyName().equals("transferHandler"))
+				{
+					if (currentDropTarget != null)
+					{
+						currentDropTarget.removeDropTargetListener(mxGraphHandler.this);
+					}
+					
+					installDropTargetHandler();
+				}
+			}
+		});
 
 		setVisible(false);
 	}
@@ -311,6 +336,7 @@ public class mxGraphHandler extends mxMouseAdapter implements
 			if (dropTarget != null)
 			{
 				dropTarget.addDropTargetListener(this);
+				currentDropTarget = dropTarget;
 			}
 		}
 		catch (TooManyListenersException tmle)
@@ -511,6 +537,22 @@ public class mxGraphHandler extends mxMouseAdapter implements
 	{
 		markerEnabled = value;
 	}
+	
+	/**
+	 * 
+	 */
+	public mxCellMarker getMarker()
+	{
+		return marker;
+	}
+	
+	/**
+	 * 
+	 */
+	public void setMarker(mxCellMarker value)
+	{
+		marker = value;
+	}
 
 	/**
 	 * 
@@ -933,8 +975,11 @@ public class mxGraphHandler extends mxMouseAdapter implements
 	{
 		// LATER: Check scrollborder, use scroll-increments, do not
 		// scroll when over ruler dragging from library
-		graphComponent.getGraphControl().scrollRectToVisible(
+		if (graphComponent.isAutoScroll())
+		{
+			graphComponent.getGraphControl().scrollRectToVisible(
 				new Rectangle(e.getPoint()));
+		}
 
 		if (!e.isConsumed())
 		{
@@ -1168,7 +1213,10 @@ public class mxGraphHandler extends mxMouseAdapter implements
 					{
 						if (tmp == null)
 						{
-							graph.clearSelection();
+							if (!graphComponent.isToggleEvent(e))
+							{
+								graph.clearSelection();
+							}
 						}
 						else if (graph.isSwimlane(tmp)
 								&& graphComponent.getCanvas()
