@@ -1,5 +1,5 @@
 /**
- * $Id: mxGraphTransferHandler.java,v 1.17 2010-07-19 11:25:20 gaudenz Exp $
+ * $Id: mxGraphTransferHandler.java,v 1.20 2011-01-25 15:56:18 gaudenz Exp $
  * Copyright (c) 2008, Gaudenz Alder
  */
 package com.mxgraph.swing.handler;
@@ -53,7 +53,13 @@ public class mxGraphTransferHandler extends TransferHandler
 	/**
 	 * Reference to the last imported cell array.
 	 */
-	protected Object[] lastImported;
+	protected Transferable lastImported;
+
+	/**
+	 * Sets the value for the initialImportCount. Default is 1. Updated in
+	 * exportDone to contain 0 after a cut and 1 after a copy.
+	 */
+	protected int initialImportCount = 1;
 
 	/**
 	 * Counter for the last imported cell array.
@@ -82,6 +88,22 @@ public class mxGraphTransferHandler extends TransferHandler
 	 */
 	protected Point offset;
 
+	/**
+	 * 
+	 */
+	public int getImportCount()
+	{
+		return importCount;
+	}
+
+	/**
+	 * 
+	 */
+	public void setImportCount(int value)
+	{
+		importCount = value;
+	}
+
 	/**
 	 * 
 	 */
@@ -175,8 +197,7 @@ public class mxGraphTransferHandler extends TransferHandler
 				if (originalCells.length > 0)
 				{
 					ImageIcon icon = (transferImageEnabled) ? createTransferableImage(
-							graphComponent, originalCells)
-							: null;
+							graphComponent, originalCells) : null;
 
 					return createGraphTransferable(graphComponent,
 							originalCells, icon);
@@ -205,8 +226,7 @@ public class mxGraphTransferHandler extends TransferHandler
 		bounds.setWidth(bounds.getWidth() / scale);
 		bounds.setHeight(bounds.getHeight() / scale);
 
-		return createGraphTransferable(graphComponent, graph.cloneCells(cells),
-				bounds, icon);
+		return createGraphTransferable(graphComponent, cells, bounds, icon);
 	}
 
 	/**
@@ -216,7 +236,8 @@ public class mxGraphTransferHandler extends TransferHandler
 			mxGraphComponent graphComponent, Object[] cells,
 			mxRectangle bounds, ImageIcon icon)
 	{
-		return new mxGraphTransferable(cells, bounds, icon);
+		return new mxGraphTransferable(graphComponent.getGraph().cloneCells(
+				cells), bounds, icon);
 	}
 
 	/**
@@ -228,9 +249,9 @@ public class mxGraphTransferHandler extends TransferHandler
 		ImageIcon icon = null;
 		Color bg = (transferImageBackground != null) ? transferImageBackground
 				: graphComponent.getBackground();
-		Image img = mxCellRenderer.createBufferedImage(graphComponent
-				.getGraph(), cells, 1, bg, graphComponent.isAntiAlias(), null,
-				graphComponent.getCanvas());
+		Image img = mxCellRenderer.createBufferedImage(
+				graphComponent.getGraph(), cells, 1, bg,
+				graphComponent.isAntiAlias(), null, graphComponent.getCanvas());
 
 		if (img != null)
 		{
@@ -245,6 +266,8 @@ public class mxGraphTransferHandler extends TransferHandler
 	 */
 	public void exportDone(JComponent c, Transferable data, int action)
 	{
+		initialImportCount = 1;
+		
 		if (c instanceof mxGraphComponent
 				&& data instanceof mxGraphTransferable)
 		{
@@ -255,6 +278,7 @@ public class mxGraphTransferHandler extends TransferHandler
 			if (action == TransferHandler.MOVE && !isLocalDrop)
 			{
 				removeCells((mxGraphComponent) c, originalCells);
+				initialImportCount = 0;
 			}
 		}
 
@@ -280,7 +304,8 @@ public class mxGraphTransferHandler extends TransferHandler
 	}
 
 	/**
-	 * 
+	 * Checks if the mxGraphTransferable data flavour is supported and calls
+	 * importGraphTransferable if possible.
 	 */
 	public boolean importData(JComponent c, Transferable t)
 	{
@@ -295,21 +320,21 @@ public class mxGraphTransferHandler extends TransferHandler
 		{
 			try
 			{
+				updateImportCount(t);
+
 				if (c instanceof mxGraphComponent)
 				{
 					mxGraphComponent graphComponent = (mxGraphComponent) c;
 
 					if (graphComponent.isEnabled()
-							&& t
-									.isDataFlavorSupported(mxGraphTransferable.dataFlavor))
+							&& t.isDataFlavorSupported(mxGraphTransferable.dataFlavor))
 					{
 						mxGraphTransferable gt = (mxGraphTransferable) t
 								.getTransferData(mxGraphTransferable.dataFlavor);
 
 						if (gt.getCells() != null)
 						{
-							result = importCells(graphComponent, gt.getCells(),
-									gt.getBounds());
+							result = importGraphTransferable(graphComponent, gt);
 						}
 
 					}
@@ -325,10 +350,27 @@ public class mxGraphTransferHandler extends TransferHandler
 	}
 
 	/**
-	 * Returns true if the cells have been imported.
+	 * Counts the number of times that the given transferable has been imported.
 	 */
-	protected boolean importCells(mxGraphComponent graphComponent,
-			Object[] cells, mxRectangle bounds)
+	protected void updateImportCount(Transferable t)
+	{
+		if (lastImported != t)
+		{
+			importCount = initialImportCount;
+		}
+		else
+		{
+			importCount++;
+		}
+
+		lastImported = t;
+	}
+
+	/**
+	 * Returns true if the cells have been imported using importCells.
+	 */
+	protected boolean importGraphTransferable(mxGraphComponent graphComponent,
+			mxGraphTransferable gt)
 	{
 		boolean result = false;
 
@@ -336,24 +378,9 @@ public class mxGraphTransferHandler extends TransferHandler
 		{
 			mxGraph graph = graphComponent.getGraph();
 			double scale = graph.getView().getScale();
+			mxRectangle bounds = gt.getBounds();
 			double dx = 0, dy = 0;
 
-			Object target = null;
-
-			// Finds the target cell at the given location and checks if the
-			// target is not already the parent of the first imported cell
-			if (location != null)
-			{
-				target = graph.getDropTarget(cells, location, graphComponent
-						.getCellAt(location.x, location.y));
-
-				if (cells.length > 0
-						&& graph.getModel().getParent(cells[0]) == target)
-				{
-					target = null;
-				}
-			}
-
 			// Computes the offset for the placement of the imported cells
 			if (location != null && bounds != null)
 			{
@@ -370,21 +397,6 @@ public class mxGraphTransferHandler extends TransferHandler
 			}
 			else
 			{
-				// Counts the number of times the cells have been imported
-				// via paste and increments the offset accordingly
-				// FIXME: Cell array loses object identify since the
-				// transferable is deserialized on each insert. We need
-				// a different way of identifying repeated inserts of the
-				// same cell array.
-				if (lastImported != cells)
-				{
-					importCount = 1;
-				}
-				else
-				{
-					importCount++;
-				}
-
 				int gs = graph.getGridSize();
 
 				dx = importCount * gs;
@@ -397,20 +409,7 @@ public class mxGraphTransferHandler extends TransferHandler
 				dy += offset.y;
 			}
 
-			lastImported = cells;
-			cells = graphComponent.getImportableCells(cells);
-
-			if (graph.isSplitEnabled() && graph.isSplitTarget(target, cells))
-			{
-				graph.splitEdge(target, cells, dx, dy);
-			}
-			else
-			{
-				cells = importCells(graphComponent, cells, dx, dy, target,
-						location);
-				graph.setSelectionCells(cells);
-			}
-
+			importCells(graphComponent, gt, dx, dy);
 			location = null;
 			offset = null;
 			result = true;
@@ -427,12 +426,58 @@ public class mxGraphTransferHandler extends TransferHandler
 	}
 
 	/**
-	 * 
+	 * Returns the drop target for the given transferable and location.
+	 */
+	protected Object getDropTarget(mxGraphComponent graphComponent,
+			mxGraphTransferable gt)
+	{
+		Object[] cells = gt.getCells();
+		Object target = null;
+
+		// Finds the target cell at the given location and checks if the
+		// target is not already the parent of the first imported cell
+		if (location != null)
+		{
+			target = graphComponent.getGraph().getDropTarget(cells, location,
+					graphComponent.getCellAt(location.x, location.y));
+
+			if (cells.length > 0
+					&& graphComponent.getGraph().getModel().getParent(cells[0]) == target)
+			{
+				target = null;
+			}
+		}
+
+		return target;
+	}
+
+	/**
+	 * Gets a drop target using getDropTarget and imports the cells using
+	 * mxGraph.splitEdge or mxGraphComponent.importCells depending on the
+	 * drop target and the return values of mxGraph.isSplitEnabled and
+	 * mxGraph.isSplitTarget. Selects and returns the cells that have been
+	 * imported.
 	 */
 	protected Object[] importCells(mxGraphComponent graphComponent,
-			Object[] cells, double dx, double dy, Object target, Point location)
+			mxGraphTransferable gt, double dx, double dy)
 	{
-		return graphComponent.importCells(cells, dx, dy, target, location);
+		Object target = getDropTarget(graphComponent, gt);
+		mxGraph graph = graphComponent.getGraph();
+		Object[] cells = gt.getCells();
+
+		cells = graphComponent.getImportableCells(cells);
+
+		if (graph.isSplitEnabled() && graph.isSplitTarget(target, cells))
+		{
+			graph.splitEdge(target, cells, dx, dy);
+		}
+		else
+		{
+			cells = graphComponent.importCells(cells, dx, dy, target, location);
+			graph.setSelectionCells(cells);
+		}
+
+		return cells;
 	}
 
 }
