@@ -1,5 +1,5 @@
 /**
- * $Id: mxGraphOutline.java,v 1.22 2010-05-31 08:49:27 gaudenz Exp $
+ * $Id: mxGraphOutline.java,v 1.25 2011-09-29 13:55:00 gaudenz Exp $
  * Copyright (c) 2008, Gaudenz Alder
  */
 package com.mxgraph.swing;
@@ -29,10 +29,10 @@ import javax.swing.JScrollBar;
 
 import com.mxgraph.util.mxEvent;
 import com.mxgraph.util.mxEventObject;
+import com.mxgraph.util.mxEventSource.mxIEventListener;
 import com.mxgraph.util.mxPoint;
 import com.mxgraph.util.mxRectangle;
 import com.mxgraph.util.mxUtils;
-import com.mxgraph.util.mxEventSource.mxIEventListener;
 import com.mxgraph.view.mxGraphView;
 
 /**
@@ -498,8 +498,8 @@ public class mxGraphOutline extends JComponent
 		{
 			if (clip == null)
 			{
-				clip = new Rectangle(tripleBuffer.getWidth(), tripleBuffer
-						.getHeight());
+				clip = new Rectangle(tripleBuffer.getWidth(),
+						tripleBuffer.getHeight());
 			}
 
 			// Clears and repaints the dirty rectangle using the
@@ -624,7 +624,14 @@ public class mxGraphOutline extends JComponent
 				// Draws the scaled page background
 				if (!graphComponent.isPageVisible())
 				{
-					g.setColor(graphComponent.getBackground());
+					Color bg = graphComponent.getBackground();
+
+					if (graphComponent.getViewport().isOpaque())
+					{
+						bg = graphComponent.getViewport().getBackground();
+					}
+
+					g.setColor(bg);
 					Dimension size = graphComponent.getGraphControl().getSize();
 
 					// Paints the background of the drawing surface
@@ -663,7 +670,9 @@ public class mxGraphOutline extends JComponent
 
 			try
 			{
-				g2.translate(translate.x, translate.y);
+				Point tr = graphComponent.getGraphControl().getTranslate();
+				g2.translate(translate.x + tr.getX() * scale,
+						translate.y + tr.getY() * scale);
 				g2.scale(scale, scale);
 
 				// Draws the scaled graph
@@ -692,13 +701,16 @@ public class mxGraphOutline extends JComponent
 			g.drawRect(finderBounds.x, finderBounds.y, finderBounds.width,
 					finderBounds.height);
 
-			g2.setStroke(stroke);
-			g.setColor(DEFAULT_ZOOMHANDLE_FILL);
-			g.fillRect(finderBounds.x + finderBounds.width - 6, finderBounds.y
-					+ finderBounds.height - 6, 8, 8);
-			g.setColor(Color.BLACK);
-			g.drawRect(finderBounds.x + finderBounds.width - 6, finderBounds.y
-					+ finderBounds.height - 6, 8, 8);
+			if (zoomHandleVisible)
+			{
+				g2.setStroke(stroke);
+				g.setColor(DEFAULT_ZOOMHANDLE_FILL);
+				g.fillRect(finderBounds.x + finderBounds.width - 6, finderBounds.y
+						+ finderBounds.height - 6, 8, 8);
+				g.setColor(Color.BLACK);
+				g.drawRect(finderBounds.x + finderBounds.width - 6, finderBounds.y
+						+ finderBounds.height - 6, 8, 8);
+			}
 		}
 	}
 
