@@ -1,11 +1,14 @@
 /**
- * $Id: mxInteractiveCanvas.java,v 1.7 2010-07-15 14:32:42 david Exp $
+ * $Id: mxInteractiveCanvas.java,v 1.9 2011-09-26 15:37:01 gaudenz Exp $
  * Copyright (c) 2007-2010, Gaudenz Alder, David Benson
  */
 package com.mxgraph.swing.view;
 
+import java.awt.Dimension;
+import java.awt.Image;
 import java.awt.Rectangle;
 import java.awt.Shape;
+import java.awt.image.ImageObserver;
 
 import com.mxgraph.canvas.mxGraphics2DCanvas;
 import com.mxgraph.shape.mxBasicShape;
@@ -18,6 +21,60 @@ import com.mxgraph.view.mxCellState;
 
 public class mxInteractiveCanvas extends mxGraphics2DCanvas
 {
+	/**
+	 * 
+	 */
+	protected ImageObserver imageObserver = null;
+
+	/**
+	 * 
+	 */
+	public mxInteractiveCanvas()
+	{
+		this(null);
+	}
+
+	/**
+	 * 
+	 */
+	public mxInteractiveCanvas(ImageObserver imageObserver)
+	{
+		setImageObserver(imageObserver);
+	}
+
+	/**
+	 * 
+	 */
+	public void setImageObserver(ImageObserver value)
+	{
+		imageObserver = value;
+	}
+
+	/**
+	 * 
+	 */
+	public ImageObserver getImageObserver()
+	{
+		return imageObserver;
+	}
+
+	/**
+	 * Overrides graphics call to use image observer.
+	 */
+	protected void drawImageImpl(Image image, int x, int y)
+	{
+		g.drawImage(image, x, y, imageObserver);
+	}
+
+	/**
+	 * Returns the size for the given image.
+	 */
+	protected Dimension getImageSize(Image image)
+	{
+		return new Dimension(image.getWidth(imageObserver),
+				image.getHeight(imageObserver));
+	}
+
 	/**
 	 * 
 	 */
@@ -113,8 +170,8 @@ public class mxInteractiveCanvas extends mxGraphics2DCanvas
 	{
 		if (swimlane != null)
 		{
-			int start = (int) Math.max(2, Math.round(mxUtils.getInt(swimlane
-					.getStyle(), mxConstants.STYLE_STARTSIZE,
+			int start = (int) Math.max(2, Math.round(mxUtils.getInt(
+					swimlane.getStyle(), mxConstants.STYLE_STARTSIZE,
 					mxConstants.DEFAULT_STARTSIZE)
 					* graphComponent.getGraph().getView().getScale()));
 			Rectangle rect = swimlane.getRectangle();
