@@ -1,5 +1,5 @@
 /**
- * $Id: mxConstants.java,v 1.62 2010-08-09 16:46:20 david Exp $
+ * $Id: mxConstants.java,v 1.71 2011-07-18 15:28:02 david Exp $
  * Copyright (c) 2007-2010, Gaudenz Alder, David Benson
  */
 package com.mxgraph.util;
@@ -73,9 +73,23 @@ public class mxConstants
 	/**
 	 * Contains an empty image of size 1, 1.
 	 */
-	public static Image EMPTY_IMAGE = new BufferedImage(1, 1,
-			BufferedImage.TYPE_INT_RGB);
+	public static BufferedImage EMPTY_IMAGE;
 
+	static
+	{
+		try
+		{
+			mxConstants.EMPTY_IMAGE = new BufferedImage(1, 1,
+					BufferedImage.TYPE_INT_RGB);
+		}
+		catch (Exception e)
+		{
+			// Occurs when running on GAE, BufferedImage is a
+			// blacklisted class
+			mxConstants.EMPTY_IMAGE = null;
+		}
+	}
+	
 	/**
 	 * Comma separated list of default fonts for CSS properties.
 	 * Default is Arial, Helvetica.
@@ -344,6 +358,14 @@ public class mxConstants
 	 */
 	public static String STYLE_TARGET_PORT = "targetPort";
 
+	/**
+	 * Defines the direction(s) that edges are allowed to connect to cells in.
+	 * Possible values are <code>DIRECTION_NORTH, DIRECTION_SOUTH, 
+	 * DIRECTION_EAST</code> and <code>DIRECTION_WEST</code>.
+	 * 
+	 */
+	public static String STYLE_PORT_CONSTRAINT = "portConstraint";
+
 	/**
 	 * Defines the key for the opacity style. The type of the value is 
 	 * <code>float</code> and the possible range is 0-100.
@@ -564,13 +586,21 @@ public class mxConstants
 	 */
 	public static String STYLE_IMAGE_VERTICAL_ALIGN = "imageVerticalAlign";
 
+	/**
+	 * Defines the key for the glass style. Possible values are 0 (disabled) and
+	 * 1(enabled). The default value is 0. This is used in mxLabel.
+	 */
+	public static String STYLE_GLASS = "glass";
+
 	/**
 	 * Defines the key for the image style. Possible values are any image URL,
-	 * the type of the value is <code>String</code>. This is the path to the
+	 * registered key in mxImageResources or short data URI as defined in
+	 * mxImageBundle.
+	 * The type of the value is <code>String</code>. This is the path to the
 	 * image to image that is to be displayed within the label of a vertex. See
 	 * mxGraphics2DCanvas.getImageForStyle, loadImage and setImageBasePath on
 	 * how the image URL is resolved. Finally, mxUtils.loadImage is used for
-	 * loading the image for a given URL.
+	 * loading the image for a given value.
 	 */
 	public static String STYLE_IMAGE = "image";
 
@@ -600,6 +630,18 @@ public class mxConstants
 	 */
 	public static String STYLE_IMAGE_BORDER = "imageBorder";
 
+	/**
+	 * Defines the key for the horizontal image flip. This style is only used
+	 * in mxImageShape. Possible values are 0 and 1. Default is 0.
+	 */
+	public static String STYLE_IMAGE_FLIPH = "imageFlipH";
+
+	/**
+	 * Defines the key for the vertical image flip. This style is only used
+	 * in mxImageShape. Possible values are 0 and 1. Default is 0.
+	 */
+	public static String STYLE_IMAGE_FLIPV = "imageFlipV";
+
 	/**
 	 * Defines the key for the noLabel style. If this is
 	 * true then no label is visible for a given cell.
@@ -866,6 +908,63 @@ public class mxConstants
 	 */
 	public static String STYLE_FONTSTYLE = "fontStyle";
 
+	/**
+	 * Defines the key for the autosize style. This specifies if a cell should be
+	 * resized automatically if the value has changed. Possible values are 0 or 1.
+	 * Default is 0. See mxGraph.isAutoSizeCell. This is normally combined with
+	 * STYLE_RESIZABLE to disable manual sizing.
+	 */
+	public static String STYLE_AUTOSIZE = "autosize";
+
+	/**
+	 * Defines the key for the foldable style. This specifies if a cell is foldable
+	 * using a folding icon. Possible values are 0 or 1. Default is 1. See
+	 * mxGraph.isCellFoldable.
+	 */
+	public static String STYLE_FOLDABLE = "foldable";
+
+	/**
+	 * Defines the key for the editable style. This specifies if the value of
+	 * a cell can be edited using the in-place editor. Possible values are 0 or
+	 * 1. Default is 1. See mxGraph.isCellEditable.
+	 */
+	public static String STYLE_EDITABLE = "editable";
+
+	/**
+	 * Defines the key for the bendable style. This specifies if the control
+	 * points of an edge can be moved. Possible values are 0 or 1. Default is
+	 * 1. See mxGraph.isCellBendable.
+	 */
+	public static String STYLE_BENDABLE = "bendable";
+
+	/**
+	 * Defines the key for the movable style. This specifies if a cell can
+	 * be moved. Possible values are 0 or 1. Default is 1. See
+	 * mxGraph.isCellMovable.
+	 */
+	public static String STYLE_MOVABLE = "movable";
+
+	/**
+	 * Defines the key for the resizable style. This specifies if a cell can
+	 * be resized. Possible values are 0 or 1. Default is 1. See
+	 * mxGraph.isCellResizable.
+	 */
+	public static String STYLE_RESIZABLE = "resizable";
+
+	/**
+	 * Defines the key for the cloneable style. This specifies if a cell can
+	 * be cloned. Possible values are 0 or 1. Default is 1. See
+	 * mxGraph.isCellCloneable.
+	 */
+	public static String STYLE_CLONEABLE = "cloneable";
+
+	/**
+	 * Defines the key for the deletable style. This specifies if a cell can be
+	 * deleted. Possible values are 0 or 1. Default is 1. See
+	 * mxGraph.isCellDeletable.
+	 */
+	public static String STYLE_DELETABLE = "deletable";
+
 	/**
 	 * Defines the key for the shape style.
 	 * Possible values are any of the SHAPE_*
@@ -1074,6 +1173,36 @@ public class mxConstants
 	 */
 	public static final String DIRECTION_WEST = "west";
 
+	/**
+	 * DIRECTION_MASK_NONE
+	 */
+	public static final int DIRECTION_MASK_NONE = 0x00;
+
+	/**
+	 * DIRECTION_MASK_WEST
+	 */
+	public static final int DIRECTION_MASK_WEST = 0x01;
+	
+	/**
+	 * DIRECTION_MASK_NORTH
+	 */
+	public static final int DIRECTION_MASK_NORTH = 0x02;
+
+	/**
+	 * DIRECTION_MASK_SOUTH
+	 */
+	public static final int DIRECTION_MASK_SOUTH = 0x04;
+
+	/**
+	 * DIRECTION_MASK_EAST
+	 */
+	public static final int DIRECTION_MASK_EAST = 0x08;
+	
+	/**
+	 * DIRECTION_MASK_EAST
+	 */
+	public static final int DIRECTION_MASK_ALL = 0x0F;
+
 	/**
 	 * ELBOW_VERTICAL
 	 */
@@ -1113,6 +1242,12 @@ public class mxConstants
 	 * for the STYLE_EDGE style.
 	 */
 	public static final String EDGESTYLE_TOPTOBOTTOM = "topToBottomEdgeStyle";
+	
+	/**
+	 * Name of the orthogonal edge style. Can be used as a string value for
+	 * the STYLE_EDGE style.
+	 */
+	public static final String EDGESTYLE_ORTHOGONAL = "orthogonalEdgeStyle";
 
 	/**
 	 * Name of the ellipse perimeter. Can be used as a string value
