@@ -1,9 +1,10 @@
 /**
- * $Id: mxCurve.java,v 1.42 2010-08-05 16:31:02 david Exp $
+ * $Id: mxCurve.java,v 1.52 2011-02-02 11:13:06 david Exp $
  * Copyright (c) 2009-2010, David Benson, Gaudenz Alder
  */
 package com.mxgraph.util;
 
+import java.awt.Rectangle;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Hashtable;
@@ -47,6 +48,12 @@ public class mxCurve
 	 */
 	public static String LABEL_CURVE = "Label_curve";;
 
+	/**
+	 * Indicates that an invalid position on a curve was requested
+	 */
+	public static mxLine INVALID_POSITION = new mxLine(new mxPoint(0, 0),
+			new mxPoint(1, 0));
+
 	/**
 	 * Offset of the label curve from the curve the label curve is based on.
 	 * If you wish to set this value, do so directly after creation of the curve.
@@ -105,7 +112,7 @@ public class mxCurve
 	{
 		boolean nullPoints = false;
 
-		for (mxPoint point : guidePoints)
+		for (mxPoint point : points)
 		{
 			if (point == null)
 			{
@@ -132,7 +139,7 @@ public class mxCurve
 		{
 			return 0;
 		}
-		
+
 		int numIntervals = curveIntervals.length;
 
 		if (distance <= 0.0 || numIntervals < 3)
@@ -224,7 +231,8 @@ public class mxCurve
 	 * @param index the curve index specifying the curve to analyse
 	 * @param distance the distance from start to end of curve (0.0...1.0)
 	 * @return a unit vector at the specified point on the curve represented
-	 * 		as a line, parallel with the curve
+	 * 		as a line, parallel with the curve. If the distance or curve is
+	 * 		invalid, <code>mxCurve.INVALID_POSITION</code> is returned
 	 */
 	public mxLine getCurveParallel(String index, double distance)
 	{
@@ -239,10 +247,9 @@ public class mxCurve
 			if (pointsCurve.length == 1)
 			{
 				mxPoint point = pointsCurve[0];
-				return new mxLine(new mxPoint(point.getX(), point.getY()),
-						new mxPoint(1, 0));
+				return new mxLine(point.getX(), point.getY(), new mxPoint(1, 0));
 			}
-			
+
 			int lowerLimit = getLowerIndexOfSegment(index, distance);
 			mxPoint firstPointOfSeg = pointsCurve[lowerLimit];
 			double segVectorX = pointsCurve[lowerLimit + 1].getX()
@@ -253,16 +260,17 @@ public class mxCurve
 					/ (curveIntervals[lowerLimit + 1] - curveIntervals[lowerLimit]);
 			double segLength = Math.sqrt(segVectorX * segVectorX + segVectorY
 					* segVectorY);
-			mxPoint startPoint = new mxPoint(firstPointOfSeg.getX()
-					+ segVectorX * distanceAlongSeg, firstPointOfSeg.getY()
-					+ segVectorY * distanceAlongSeg);
+			double startPointX = firstPointOfSeg.getX() + segVectorX
+					* distanceAlongSeg;
+			double startPointY = firstPointOfSeg.getY() + segVectorY
+					* distanceAlongSeg;
 			mxPoint endPoint = new mxPoint(segVectorX / segLength, segVectorY
 					/ segLength);
-			return new mxLine(startPoint, endPoint);
+			return new mxLine(startPointX, startPointY, endPoint);
 		}
 		else
 		{
-			return new mxLine(new mxPoint(0, 0), new mxPoint(1, 0));
+			return INVALID_POSITION;
 		}
 	}
 
@@ -349,12 +357,11 @@ public class mxCurve
 	 * @param rect the rectangle to detect for a hit
 	 * @return whether or not the rectangle hits this curve
 	 */
-	public boolean intersectsRect(mxRectangle rect)
+	public boolean intersectsRect(Rectangle rect)
 	{
 		// To save CPU, we can test if the rectangle intersects the entire
 		// bounds of this curve
-		//if (this.getBounds().intersectsRect(rect))
-		if (!getBounds().getRectangle().intersects(rect.getRectangle()))
+		if (!getBounds().getRectangle().intersects(rect))
 		{
 			return false;
 		}
@@ -363,13 +370,15 @@ public class mxCurve
 
 		if (pointsCurve != null && pointsCurve.length > 1)
 		{
+			mxRectangle mxRect = new mxRectangle(rect);
 			// First check for any of the curve points lying within the 
 			// rectangle, then for any of the curve segments intersecting 
 			// with the rectangle sides
 			for (int i = 1; i < pointsCurve.length; i++)
 			{
-				if (rect.contains(pointsCurve[i].getX(), pointsCurve[i].getY())
-						|| rect.contains(pointsCurve[i - 1].getX(),
+				if (mxRect.contains(pointsCurve[i].getX(),
+						pointsCurve[i].getY())
+						|| mxRect.contains(pointsCurve[i - 1].getX(),
 								pointsCurve[i - 1].getY()))
 				{
 					return true;
@@ -378,7 +387,7 @@ public class mxCurve
 
 			for (int i = 1; i < pointsCurve.length; i++)
 			{
-				if (rect.intersectLine(pointsCurve[i].getX(),
+				if (mxRect.intersectLine(pointsCurve[i].getX(),
 						pointsCurve[i].getY(), pointsCurve[i - 1].getX(),
 						pointsCurve[i - 1].getY()) != null)
 				{
@@ -424,7 +433,7 @@ public class mxCurve
 	/**
 	 * Returns the distance from the start of the curve at which this 
 	 * curve intersects the boundary of the given rectangle, if it does 
-	 * so. If it does not intersect, a negative value is returned. 
+	 * so. If it does not intersect, -1 is returned. 
 	 * If it intersects multiple times, the first intersection from 
 	 * the start end of the curve is returned.
 	 * 
@@ -469,6 +478,81 @@ public class mxCurve
 		return result;
 	}
 
+	/**
+	 * Returns a point to move the input rectangle to, in order to
+	 * attempt to place the rectangle away from the curve. NOTE: Curves
+	 * are scaled, the input rectangle should be also.
+	 * @param index  the curve index specifying the curve to analyse
+	 * @param rect the rectangle that is to be moved
+	 * @param buffer the amount by which the rectangle is to be moved,
+	 * 			beyond the dimensions of the rect
+	 * @return the point to move the top left of the input rect to
+	 * 			, otherwise null if no point can be determined
+	 */
+	public mxPoint collisionMove(String index, mxRectangle rect, double buffer)
+	{
+		int hitSeg = intersectRectPerimeterSeg(index, rect);
+
+		// Could test for a second hit (the rect exit, unless the same 
+		// segment is entry and exit) and allow for that in movement.
+		
+		if (hitSeg == -1)
+		{
+			return null;
+		}
+		else
+		{
+			mxPoint[] pointsCurve = getCurvePoints(index);
+
+			double x0 = pointsCurve[hitSeg - 1].getX();
+			double y0 = pointsCurve[hitSeg - 1].getY();
+			double x1 = pointsCurve[hitSeg].getX();
+			double y1 = pointsCurve[hitSeg].getY();
+			
+			double x = rect.getX();
+			double y = rect.getY();
+			double width = rect.getWidth();
+			double height = rect.getHeight();
+
+			// Whether the intersection is one of the horizontal sides of the rect
+			boolean horizIncident = false;
+			mxPoint hitPoint = mxUtils.intersection(x, y, x + width, y, x0, y0, x1, y1);
+			
+			if (hitPoint != null)
+			{
+				horizIncident = true;
+			}
+			else
+			{
+				hitPoint = mxUtils.intersection(x + width, y, x + width, y + height,
+						x0, y0, x1, y1);
+			}
+
+			if (hitPoint == null)
+			{
+				hitPoint = mxUtils.intersection(x + width, y + height, x, y + height,
+						x0, y0, x1, y1);
+				
+				if (hitPoint != null)
+				{
+					horizIncident = true;
+				}
+				else
+				{
+					hitPoint = mxUtils.intersection(x, y, x, y + height, x0, y0, x1, y1);
+				}
+			}
+
+			if (hitPoint != null)
+			{
+				
+			}
+
+		}
+		
+		return null;
+	}
+
 	/**
 	 * Utility method to determine within which segment the specified rectangle
 	 * intersects the specified curve
@@ -481,12 +565,32 @@ public class mxCurve
 	 * -1 is returned
 	 */
 	protected int intersectRectPerimeterSeg(String index, mxRectangle rect)
+	{
+		return intersectRectPerimeterSeg(index, rect, 1);
+	}
+
+	/**
+	 * Utility method to determine within which segment the specified rectangle
+	 * intersects the specified curve. This method specifies which segment to
+	 * start searching at.
+	 * 
+	 * @param index the curve index specifying the curve to analyse
+	 * @param rect the whose boundary is to be tested for intersection
+	 * with this curve
+	 * @param startSegment the segment to start searching at. To start at the 
+	 * 			beginning of the curve, use 1, not 0.
+	 * @return the point at which this curve intersects the boundary of 
+	 * the given rectangle, if it does so. If it does not intersect, 
+	 * -1 is returned
+	 */
+	protected int intersectRectPerimeterSeg(String index, mxRectangle rect,
+			int startSegment)
 	{
 		mxPoint[] pointsCurve = getCurvePoints(index);
 
 		if (pointsCurve != null && pointsCurve.length > 1)
 		{
-			for (int i = 1; i < pointsCurve.length; i++)
+			for (int i = startSegment; i < pointsCurve.length; i++)
 			{
 				if (rect.intersectLine(pointsCurve[i].getX(),
 						pointsCurve[i].getY(), pointsCurve[i - 1].getX(),
@@ -542,7 +646,7 @@ public class mxCurve
 	 * calculated from
 	 * @return an mxRectangle where the x is the distance along the curve 
 	 * (0 to 1), y is the orthogonal offset from the closest segment on the 
-	 * curve and (wdith, height) is an additional cartisian offset applied
+	 * curve and (width, height) is an additional Cartesian offset applied
 	 * after the other calculations
 	 */
 	public mxRectangle getRelativeFromAbsPoint(mxPoint absPoint, String index)
@@ -552,10 +656,11 @@ public class mxCurve
 		double[] currentIntervals = getIntervals(index);
 		int closestSegment = 0;
 		double closestSegDistSq = 10000000;
+		mxLine segment = new mxLine(currentCurve[0], currentCurve[1]);
 
 		for (int i = 1; i < currentCurve.length; i++)
 		{
-			mxLine segment = new mxLine(currentCurve[i - 1], currentCurve[i]);
+			segment.setPoints(currentCurve[i - 1], currentCurve[i]);
 			double segDistSq = segment.ptSegDistSq(absPoint);
 
 			if (segDistSq < closestSegDistSq)
@@ -675,7 +780,8 @@ public class mxCurve
 	}
 
 	/**
-	 * Creates the actual 
+	 * Creates the core curve that is based on the guide points passed into
+	 * this class instance
 	 */
 	protected void createCoreCurve()
 	{
@@ -891,17 +997,6 @@ public class mxCurve
 		intervals.put(CORE_CURVE, coreIntervalsArray);
 
 		valid = true;
-
-		mxPoint[] centralCurve = getCurvePoints(CORE_CURVE);
-		if (centralCurve == null)
-		{
-			System.out.println("oh bugger");
-		}
-	}
-
-	public void configureLabelCurve()
-	{
-
 	}
 
 	/** Whether or not the label curve starts from the end target
@@ -941,7 +1036,7 @@ public class mxCurve
 
 		// Lower and upper curve start from the very ends
 		// of their curves, so given that their middle points
-		// are dervied from the center of the central points
+		// are derived from the center of the central points
 		// they will contain one more point and both
 		// side curves contain the same end point
 
@@ -981,7 +1076,7 @@ public class mxCurve
 			double pointX = centerSegX - (normSegVectorY * labelBuffer);
 			double pointY = centerSegY + (normSegVectorX * labelBuffer);
 			mxPoint labelCurvePoint = new mxPoint(pointX, pointY);
-			updateBounds(pointX, pointX);
+			updateBounds(pointX, pointY);
 			labelCurvePoints.add(labelCurvePoint);
 
 			if (i == currentCurve.length - 1)
@@ -1074,24 +1169,24 @@ public class mxCurve
 		}
 		else
 		{
-			if (!guidePoints.equals(newPoints))
-			{
-				pointsChanged = true;
-			}
-
 			// Check for a constant translation of all guide points. In that 
-			// case apply the translation directly to all curves
-			if (valid && pointsChanged
-					&& newPoints.size() == guidePoints.size()
-					&& newPoints.size() > 1 && guidePoints.size() > 1)
+			// case apply the translation directly to all curves.
+			// Also check whether all of the translations are trivial
+			if (newPoints.size() == guidePoints.size() && newPoints.size() > 1
+					&& guidePoints.size() > 1)
 			{
-
 				boolean constantTranslation = true;
+				boolean trivialTranslation = true;
 				mxPoint newPoint0 = newPoints.get(0);
 				mxPoint oldPoint0 = guidePoints.get(0);
 				double transX = newPoint0.getX() - oldPoint0.getX();
 				double transY = newPoint0.getY() - oldPoint0.getY();
 
+				if (Math.abs(transX) > 0.01 || Math.abs(transY) > 0.01)
+				{
+					trivialTranslation = false;
+				}
+
 				for (int i = 1; i < newPoints.size(); i++)
 				{
 					double nextTransX = newPoints.get(i).getX()
@@ -1099,14 +1194,24 @@ public class mxCurve
 					double nextTransY = newPoints.get(i).getY()
 							- guidePoints.get(i).getY();
 
-					if (transX != nextTransX || transY != nextTransY)
+					if (Math.abs(transX - nextTransX) > 0.01
+							|| Math.abs(transY - nextTransY) > 0.01)
 					{
 						constantTranslation = false;
-						break;
+					}
+
+					if (Math.abs(nextTransX) > 0.01
+							|| Math.abs(nextTransY) > 0.01)
+					{
+						trivialTranslation = false;
 					}
 				}
 
-				if (constantTranslation)
+				if (trivialTranslation)
+				{
+					pointsChanged = false;
+				}
+				else if (constantTranslation)
 				{
 					pointsChanged = false;
 					// Translate all stored points by the translation amounts
@@ -1130,6 +1235,10 @@ public class mxCurve
 					maxXBounds += transX;
 					maxYBounds += transY;
 				}
+				else
+				{
+					pointsChanged = true;
+				}
 			}
 		}
 
