@@ -1,6 +1,6 @@
 /**
- * $Id: mxRectangle.java,v 1.11 2010-05-28 13:57:32 gaudenz Exp $
- * Copyright (c) 2007, Gaudenz Alder
+ * $Id: mxRectangle.java,v 1.14 2010-12-08 12:46:03 gaudenz Exp $
+ * Copyright (c) 2007-2010, Gaudenz Alder, David Benson
  */
 package com.mxgraph.util;
 
@@ -107,6 +107,22 @@ public class mxRectangle extends mxPoint
 		height = value;
 	}
 
+	/**
+	 * Sets this rectangle to the specified values
+	 * 
+	 * @param x the new x-axis position
+	 * @param y the new y-axis position
+	 * @param w the new width of the rectangle
+	 * @param h the new height of the rectangle
+	 */
+	public void setRect(double x, double y, double w, double h)
+	{
+	    this.x = x;
+	    this.y = y;
+	    this.width = w;
+	    this.height = h;
+	}
+
 	/**
 	 * Adds the given rectangle to this rectangle.
 	 */
@@ -257,4 +273,15 @@ public class mxRectangle extends mxPoint
 		return clone;
 	}
 
+	/**
+	 * Returns the <code>String</code> representation of this
+	 * <code>mxRectangle</code>.
+	 * @return a <code>String</code> representing this
+	 * <code>mxRectangle</code>.
+	 */
+	public String toString()
+	{
+		return getClass().getName() + "[x=" + x + ",y=" + y + ",w=" + width
+				+ ",h=" + height + "]";
+	}
 }
