@@ -1,6 +1,6 @@
 /**
- * $Id: mxUtils.java,v 1.92 2010-09-07 20:20:21 david Exp $
- * Copyright (c) 2007-2010, Gaudenz Alder, David Benson
+ * $Id: mxUtils.java,v 1.113 2011-07-25 20:45:46 gaudenz Exp $
+ * Copyright (c) 2007-2011, Gaudenz Alder, David Benson
  */
 package com.mxgraph.util;
 
@@ -11,7 +11,6 @@ import java.awt.Font;
 import java.awt.FontMetrics;
 import java.awt.Graphics;
 import java.awt.Graphics2D;
-import java.awt.Image;
 import java.awt.Rectangle;
 import java.awt.RenderingHints;
 import java.awt.font.FontRenderContext;
@@ -20,12 +19,13 @@ import java.awt.geom.Rectangle2D;
 import java.awt.image.BufferedImage;
 import java.awt.image.ImageObserver;
 import java.io.BufferedReader;
-import java.io.ByteArrayOutputStream;
+import java.io.ByteArrayInputStream;
 import java.io.FileInputStream;
 import java.io.FileWriter;
 import java.io.IOException;
 import java.io.InputStreamReader;
 import java.io.StringReader;
+import java.io.StringWriter;
 import java.net.URL;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
@@ -45,8 +45,7 @@ import javax.imageio.ImageIO;
 import javax.swing.text.html.HTMLDocument;
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
-import javax.xml.transform.Result;
-import javax.xml.transform.Source;
+import javax.xml.transform.OutputKeys;
 import javax.xml.transform.Transformer;
 import javax.xml.transform.TransformerFactory;
 import javax.xml.transform.dom.DOMSource;
@@ -74,6 +73,12 @@ import com.mxgraph.view.mxCellState;
 public class mxUtils
 {
 
+	/**
+	 * True if the machine is a Mac.
+	 */
+	public static boolean IS_MAC = System.getProperty("os.name").toLowerCase()
+			.indexOf("mac") >= 0;
+
 	/**
 	 * Static Graphics used for Font Metrics.
 	 */
@@ -100,17 +105,17 @@ public class mxUtils
 	 * body are converted into linebreaks.
 	 */
 	public static mxRectangle getLabelSize(String label,
-			Map<String, Object> style, boolean isHtml, int width)
+			Map<String, Object> style, boolean isHtml, double scale)
 	{
 		mxRectangle size;
 
 		if (isHtml)
 		{
-			size = getSizeForHtml(getBodyMarkup(label, true), style);
+			size = getSizeForHtml(getBodyMarkup(label, true), style, scale);
 		}
 		else
 		{
-			size = getSizeForString(label, getFont(style), width);
+			size = getSizeForString(label, getFont(style), scale);
 		}
 
 		return size;
@@ -150,39 +155,11 @@ public class mxUtils
 			Map<String, Object> style, boolean isHtml, mxPoint offset,
 			mxRectangle vertexBounds, double scale)
 	{
-		boolean horizontal = mxUtils.isTrue(style,
-				mxConstants.STYLE_HORIZONTAL, true);
-		int w = 0;
+		mxRectangle size = mxUtils.getLabelSize(label, style, isHtml, scale);
 
-		if (vertexBounds != null
-				&& getString(style, mxConstants.STYLE_WHITE_SPACE, "nowrap")
-						.equals("wrap"))
-		{
-			if (horizontal)
-			{
-				w = (int) ((vertexBounds.getWidth() / scale)
-						- 2
-						* mxConstants.LABEL_INSET
-						- 2
-						* mxUtils.getDouble(style, mxConstants.STYLE_SPACING)
-						- mxUtils.getDouble(style,
-								mxConstants.STYLE_SPACING_LEFT) - mxUtils
-						.getDouble(style, mxConstants.STYLE_SPACING_RIGHT));
-			}
-			else
-			{
-				w = (int) ((vertexBounds.getHeight() / scale)
-						- 2
-						* mxConstants.LABEL_INSET
-						- 2
-						* mxUtils.getDouble(style, mxConstants.STYLE_SPACING)
-						- mxUtils.getDouble(style,
-								mxConstants.STYLE_SPACING_TOP) + mxUtils
-						.getDouble(style, mxConstants.STYLE_SPACING_BOTTOM));
-			}
-		}
-
-		mxRectangle size = mxUtils.getLabelSize(label, style, isHtml, w);
+		// Measures font with full scale and scales back
+		size.setWidth(size.getWidth() / scale);
+		size.setHeight(size.getHeight() / scale);
 
 		double x = offset.getX();
 		double y = offset.getY();
@@ -198,6 +175,8 @@ public class mxUtils
 					mxConstants.SHAPE_SWIMLANE))
 			{
 				// Limits the label to the swimlane title
+				boolean horizontal = mxUtils.isTrue(style,
+						mxConstants.STYLE_HORIZONTAL, true);
 				double start = mxUtils.getDouble(style,
 						mxConstants.STYLE_STARTSIZE,
 						mxConstants.DEFAULT_STARTSIZE)
@@ -310,6 +289,21 @@ public class mxUtils
 		return new mxRectangle(x, y, width, height);
 	}
 
+	/**
+	 * Returns the font metrics of the static font graphics instance
+	 * @param font The font whose metrics are to be returned
+	 * @return the font metrics of the specified font
+	 */
+	public static FontMetrics getFontMetrics(Font font)
+	{
+		if (fontGraphics != null)
+		{
+			return fontGraphics.getFontMetrics(font);
+		}
+
+		return null;
+	}
+
 	/**
 	 * Returns an <mxRectangle> with the size (width and height in pixels) of
 	 * the given string.
@@ -319,9 +313,11 @@ public class mxUtils
 	 * @param font
 	 *            Font to be used for the computation.
 	 */
-	public static mxRectangle getSizeForString(String text, Font font, int width)
+	public static mxRectangle getSizeForString(String text, Font font,
+			double scale)
 	{
 		FontRenderContext frc = new FontRenderContext(null, false, false);
+		font = font.deriveFont((float) (font.getSize2D() * scale));
 		FontMetrics metrics = null;
 
 		if (fontGraphics != null)
@@ -340,24 +336,7 @@ public class mxUtils
 			lineHeight += font.getSize2D() * 1.27;
 		}
 
-		String[] lines;
-
-		if (width > 0)
-		{
-			// The lines for wrapping within the given width are calcuated for no
-			// scale. The reason for this is the granularity of actual displayed 
-			// font can cause the displayed lines to change based on scale. A factor 
-			// is used to allow for different overalls widths, it ensures the largest 
-			// font size/scale factor still stays within the bounds. All this ensures
-			// the wrapped lines are constant overing scaling, at the expense the 
-			// label bounds will vary.
-			lines = mxUtils.wordWrap(text, metrics,
-					(int) (width * mxConstants.LABEL_SCALE_BUFFER));
-		}
-		else
-		{
-			lines = text.split("\n");
-		}
+		String[] lines = text.split("\n");
 
 		Rectangle2D boundingBox = null;
 
@@ -371,9 +350,9 @@ public class mxUtils
 			}
 			else
 			{
-				boundingBox.setFrame(0, 0, Math.max(boundingBox.getWidth(),
-						bounds.getWidth()), boundingBox.getHeight()
-						+ lineHeight);
+				boundingBox.setFrame(0, 0,
+						Math.max(boundingBox.getWidth(), bounds.getWidth()),
+						boundingBox.getHeight() + lineHeight);
 			}
 		}
 
@@ -388,7 +367,8 @@ public class mxUtils
 	 * @param width the width that the text must fit within
 	 * @return the input text split in lines that fit the specified width
 	 */
-	public static String[] wordWrap(String text, FontMetrics metrics, int width)
+	public static String[] wordWrap(String text, FontMetrics metrics,
+			double width)
 	{
 		List<String> result = new ArrayList<String>();
 		// First split the processing into lines already delimited by
@@ -426,7 +406,7 @@ public class mxUtils
 
 				if (word.length() > 0)
 				{
-					// Concatenate any preceeding whitespace to the
+					// Concatenate any preceding whitespace to the
 					// word and calculate the number of characters of that
 					// whitespace
 					char firstWordLetter = word.charAt(0);
@@ -546,7 +526,7 @@ public class mxUtils
 	 *            HTML markup whose size should be returned.
 	 */
 	public static mxRectangle getSizeForHtml(String markup,
-			Map<String, Object> style)
+			Map<String, Object> style, double scale)
 	{
 		mxLightweightLabel textRenderer = mxLightweightLabel
 				.getSharedInstance();
@@ -556,11 +536,12 @@ public class mxUtils
 			textRenderer.setText(createHtmlDocument(style, markup));
 			Dimension size = textRenderer.getPreferredSize();
 
-			return new mxRectangle(0, 0, size.width, size.height);
+			return new mxRectangle(0, 0, size.width * scale, size.height
+					* scale);
 		}
 		else
 		{
-			return getSizeForString(markup, getFont(style), 0);
+			return getSizeForString(markup, getFont(style), scale);
 		}
 	}
 
@@ -577,9 +558,8 @@ public class mxUtils
 			double cos = Math.cos(rad);
 			double sin = Math.sin(rad);
 
-			mxPoint cx = new mxPoint(rect.getX() + rect.getWidth() / 2, rect
-					.getY()
-					+ rect.getHeight() / 2);
+			mxPoint cx = new mxPoint(rect.getX() + rect.getWidth() / 2,
+					rect.getY() + rect.getHeight() / 2);
 
 			mxPoint p1 = new mxPoint(rect.getX(), rect.getY());
 			mxPoint p2 = new mxPoint(rect.getX() + rect.getWidth(), rect.getY());
@@ -607,6 +587,55 @@ public class mxUtils
 		return result;
 	}
 
+	/**
+	 * Find the first character matching the input character in the given
+	 * string where the character has no letter preceding it.
+	 * 
+	 * @param text the string to test for the presence of the input character
+	 * @param inputChar the test character
+	 * @param fromIndex the index position of the string to start from
+	 * @return 
+	 */
+	public static int firstCharAt(String text, int inputChar, int fromIndex)
+	{
+		int result = 0;
+
+		while (result >= 0)
+		{
+			result = text.indexOf(inputChar, fromIndex);
+
+			if (result == 0)
+			{
+				return result;
+			}
+			else if (result > 0)
+			{
+				// Check there is a whitespace or symbol before the hit character
+				if (Character.isLetter(text.codePointAt(result - 1)))
+				{
+					// The pre-increment is used in if and else branches.
+					if (++fromIndex >= text.length())
+					{
+						return -1;
+					}
+					else
+					{
+						// Test again from next candidate character
+						// This isn't the first letter of this word
+						result = text.indexOf(inputChar, fromIndex);
+					}
+				}
+				else
+				{
+					return result;
+				}
+			}
+
+		}
+
+		return result;
+	}
+
 	/**
 	 * Rotates the given point by the given cos and sin.
 	 */
@@ -632,7 +661,7 @@ public class mxUtils
 			{
 				mxPoint current = state.getAbsolutePoint(i);
 				double dist = new Line2D.Double(last.x, last.y, current.x,
-						current.y).ptLineDist(x, y);
+						current.y).ptSegDistSq(x, y);
 
 				if (dist < min)
 				{
@@ -662,6 +691,72 @@ public class mxUtils
 		return new mxPoint(x1 + c.getX(), y1 + c.getY());
 	}
 
+	/**
+	 * Returns an integer mask of the port constraints of the given map
+	 * @param dict the style map to determine the port constraints for
+	 * @return the mask of port constraint directions
+	 */
+	public static int getPortConstraints(mxCellState terminal,
+			mxCellState edge, boolean source)
+	{
+		return getPortConstraints(terminal, edge, source,
+				mxConstants.DIRECTION_MASK_ALL);
+	}
+
+	/**
+	 * Returns an integer mask of the port constraints of the given map
+	 * @param dict the style map to determine the port constraints for
+	 * @param defaultValue Default value to return if the key is undefined.
+	 * @return the mask of port constraint directions
+	 */
+	public static int getPortConstraints(mxCellState terminal,
+			mxCellState edge, boolean source, int defaultValue)
+	{
+		Object value = terminal.getStyle().get(
+				mxConstants.STYLE_PORT_CONSTRAINT);
+
+		if (value == null)
+		{
+			return defaultValue;
+		}
+		else
+		{
+			String directions = value.toString();
+			int returnValue = mxConstants.DIRECTION_MASK_NONE;
+
+			if (directions.indexOf(mxConstants.DIRECTION_NORTH) >= 0)
+			{
+				returnValue |= mxConstants.DIRECTION_MASK_NORTH;
+			}
+			if (directions.indexOf(mxConstants.DIRECTION_WEST) >= 0)
+			{
+				returnValue |= mxConstants.DIRECTION_MASK_WEST;
+			}
+			if (directions.indexOf(mxConstants.DIRECTION_SOUTH) >= 0)
+			{
+				returnValue |= mxConstants.DIRECTION_MASK_SOUTH;
+			}
+			if (directions.indexOf(mxConstants.DIRECTION_EAST) >= 0)
+			{
+				returnValue |= mxConstants.DIRECTION_MASK_EAST;
+			}
+
+			return returnValue;
+		}
+	}
+
+	public static int reversePortConstraints(int constraint)
+	{
+		int result = 0;
+
+		result = (constraint & mxConstants.DIRECTION_MASK_WEST) << 3;
+		result |= (constraint & mxConstants.DIRECTION_MASK_NORTH) << 1;
+		result |= (constraint & mxConstants.DIRECTION_MASK_SOUTH) >> 1;
+		result |= (constraint & mxConstants.DIRECTION_MASK_EAST) >> 3;
+
+		return result;
+	}
+
 	/**
 	 * Draws the image inside the clip bounds to the given graphics object.
 	 */
@@ -1906,6 +2001,144 @@ public class mxUtils
 		return result;
 	}
 
+	/**
+	 * Loads an image from the local filesystem, a data URI or any other URL.
+	 */
+	public static BufferedImage loadImage(String url)
+	{
+		BufferedImage img = null;
+
+		if (url != null)
+		{
+			// Parses data URIs of the form data:image/format;base64,xxx
+			if (url.startsWith("data:image/"))
+			{
+				try
+				{
+					int comma = url.indexOf(',');
+					byte[] data = mxBase64.decode(url.substring(comma + 1));
+					ByteArrayInputStream is = new ByteArrayInputStream(data);
+					img = ImageIO.read(is);
+				}
+				catch (Exception e1)
+				{
+					// ignore
+				}
+			}
+			else
+			{
+				URL realUrl = null;
+
+				try
+				{
+					realUrl = new URL(url);
+				}
+				catch (Exception e)
+				{
+					realUrl = mxUtils.class.getResource(url);
+				}
+
+				if (realUrl != null)
+				{
+					try
+					{
+						img = ImageIO.read(realUrl);
+					}
+					catch (Exception e1)
+					{
+						// ignore
+					}
+				}
+			}
+		}
+
+		return img;
+	}
+
+	/**
+	 * Creates a table for the given text using the given document to create the
+	 * DOM nodes. Returns the outermost table node.
+	 */
+	public static Element createTable(Document document, String text, int x,
+			int y, int w, int h, double scale, Map<String, Object> style)
+	{
+		// Does not use a textbox as this must go inside another VML shape
+		Element table = document.createElement("table");
+
+		if (text != null && text.length() > 0)
+		{
+			Element tr = document.createElement("tr");
+			Element td = document.createElement("td");
+
+			table.setAttribute("cellspacing", "0");
+			table.setAttribute("border", "0");
+			td.setAttribute("align", mxUtils.getString(style,
+					mxConstants.STYLE_ALIGN, mxConstants.ALIGN_CENTER));
+
+			String fontColor = getString(style, mxConstants.STYLE_FONTCOLOR,
+					"black");
+			String fontFamily = getString(style, mxConstants.STYLE_FONTFAMILY,
+					mxConstants.DEFAULT_FONTFAMILIES);
+			int fontSize = (int) (getInt(style, mxConstants.STYLE_FONTSIZE,
+					mxConstants.DEFAULT_FONTSIZE) * scale);
+
+			String s = "position:absolute;" + "left:" + String.valueOf(x)
+					+ "px;" + "top:" + String.valueOf(y) + "px;" + "width:"
+					+ String.valueOf(w) + "px;" + "height:" + String.valueOf(h)
+					+ "px;" + "font-size:" + String.valueOf(fontSize) + "px;"
+					+ "font-family:" + fontFamily + ";" + "color:" + fontColor
+					+ ";";
+
+			if (mxUtils.getString(style, mxConstants.STYLE_WHITE_SPACE,
+					"nowrap").equals("wrap"))
+			{
+				s += "whiteSpace:wrap;";
+			}
+
+			// Applies the background color
+			String background = getString(style,
+					mxConstants.STYLE_LABEL_BACKGROUNDCOLOR);
+
+			if (background != null)
+			{
+				s += "background:" + background + ";";
+			}
+
+			// Applies the border color
+			String border = getString(style,
+					mxConstants.STYLE_LABEL_BORDERCOLOR);
+
+			if (border != null)
+			{
+				s += "border:" + border + " solid 1pt;";
+			}
+
+			// Applies the opacity
+			float opacity = getFloat(style, mxConstants.STYLE_TEXT_OPACITY, 100);
+
+			if (opacity < 100)
+			{
+				// Adds all rules (first for IE)
+				s += "filter:alpha(opacity=" + opacity + ");";
+				s += "opacity:" + (opacity / 100) + ";";
+			}
+
+			td.setAttribute("style", s);
+			String[] lines = text.split("\n");
+
+			for (int i = 0; i < lines.length; i++)
+			{
+				td.appendChild(document.createTextNode(lines[i]));
+				td.appendChild(document.createElement("br"));
+			}
+
+			tr.appendChild(td);
+			table.appendChild(tr);
+		}
+
+		return table;
+	}
+
 	/**
 	 * Returns a new, empty DOM document.
 	 * 
@@ -1971,9 +2204,8 @@ public class mxUtils
 
 		Element style = document.createElement("style");
 		style.setAttribute("type", "text/css");
-		style
-				.appendChild(document
-						.createTextNode("<!-- v\\:* {behavior: url(#default#VML);} -->"));
+		style.appendChild(document
+				.createTextNode("<!-- v\\:* {behavior: url(#default#VML);} -->"));
 
 		head.appendChild(style);
 		root.appendChild(head);
@@ -2132,122 +2364,6 @@ public class mxUtils
 		return document;
 	}
 
-	/**
-	 * Creates a table for the given text using the given document to create the
-	 * DOM nodes. Returns the outermost table node.
-	 */
-	public static Element createTable(Document document, String text, int x,
-			int y, int w, int h, double scale, Map<String, Object> style)
-	{
-		// Does not use a textbox as this must go inside another VML shape
-		Element table = document.createElement("table");
-
-		if (text != null && text.length() > 0)
-		{
-			Element tr = document.createElement("tr");
-			Element td = document.createElement("td");
-
-			table.setAttribute("cellspacing", "0");
-			table.setAttribute("border", "0");
-			td.setAttribute("align", mxUtils.getString(style,
-					mxConstants.STYLE_ALIGN, mxConstants.ALIGN_CENTER));
-
-			String fontColor = getString(style, mxConstants.STYLE_FONTCOLOR,
-					"black");
-			String fontFamily = getString(style, mxConstants.STYLE_FONTFAMILY,
-					mxConstants.DEFAULT_FONTFAMILIES);
-			int fontSize = (int) (getInt(style, mxConstants.STYLE_FONTSIZE,
-					mxConstants.DEFAULT_FONTSIZE) * scale);
-
-			String s = "position:absolute;" + "left:" + String.valueOf(x)
-					+ "px;" + "top:" + String.valueOf(y) + "px;" + "width:"
-					+ String.valueOf(w) + "px;" + "height:" + String.valueOf(h)
-					+ "px;" + "font-size:" + String.valueOf(fontSize) + "px;"
-					+ "font-family:" + fontFamily + ";" + "color:" + fontColor
-					+ ";";
-
-			if (mxUtils.getString(style, mxConstants.STYLE_WHITE_SPACE,
-					"nowrap").equals("wrap"))
-			{
-				s += "whiteSpace:wrap;";
-			}
-
-			// Applies the background color
-			String background = getString(style,
-					mxConstants.STYLE_LABEL_BACKGROUNDCOLOR);
-
-			if (background != null)
-			{
-				s += "background:" + background + ";";
-			}
-
-			// Applies the border color
-			String border = getString(style,
-					mxConstants.STYLE_LABEL_BORDERCOLOR);
-
-			if (border != null)
-			{
-				s += "border:" + border + " solid 1pt;";
-			}
-
-			// Applies the opacity
-			float opacity = getFloat(style, mxConstants.STYLE_TEXT_OPACITY, 100);
-
-			if (opacity < 100)
-			{
-				// Adds all rules (first for IE)
-				s += "filter:alpha(opacity=" + opacity + ");";
-				s += "opacity:" + (opacity / 100) + ";";
-			}
-
-			td.setAttribute("style", s);
-			String[] lines = text.split("\n");
-
-			for (int i = 0; i < lines.length; i++)
-			{
-				td.appendChild(document.createTextNode(lines[i]));
-				td.appendChild(document.createElement("br"));
-			}
-
-			tr.appendChild(td);
-			table.appendChild(tr);
-		}
-
-		return table;
-	}
-
-	/**
-	 * 
-	 */
-	public static Image loadImage(String url)
-	{
-		Image img = null;
-		URL realUrl = null;
-
-		try
-		{
-			realUrl = new URL(url);
-		}
-		catch (Exception e)
-		{
-			realUrl = mxUtils.class.getResource(url);
-		}
-
-		if (url != null)
-		{
-			try
-			{
-				img = ImageIO.read(realUrl);
-			}
-			catch (Exception e1)
-			{
-				// ignore
-			}
-		}
-
-		return img;
-	}
-
 	/**
 	 * Returns a new DOM document for the given URI.
 	 * 
@@ -2321,8 +2437,8 @@ public class mxUtils
 
 		if (dot > 0)
 		{
-			Class<?> clazz = mxCodecRegistry.getClassForName(expression.substring(
-					0, dot));
+			Class<?> clazz = mxCodecRegistry.getClassForName(expression
+					.substring(0, dot));
 
 			if (clazz != null)
 			{
@@ -2404,8 +2520,8 @@ public class mxUtils
 	public static String htmlEntities(String text)
 	{
 		return text.replaceAll("&", "&amp;").replaceAll("\"", "&quot;")
-				.replaceAll("'", "&prime;").replaceAll("<", "&lt;").replaceAll(
-						">", "&gt;");
+				.replaceAll("'", "&prime;").replaceAll("<", "&lt;")
+				.replaceAll(">", "&gt;");
 	}
 
 	/**
@@ -2419,15 +2535,15 @@ public class mxUtils
 	{
 		try
 		{
-			TransformerFactory tranFactory = TransformerFactory.newInstance();
-			Transformer aTransformer = tranFactory.newTransformer();
+			Transformer tf = TransformerFactory.newInstance().newTransformer();
 
-			Source src = new DOMSource(node);
-			ByteArrayOutputStream stream = new ByteArrayOutputStream();
-			Result dest = new StreamResult(stream);
-			aTransformer.transform(src, dest);
+			tf.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
+			tf.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
 
-			return stream.toString("UTF-8");
+			StreamResult dest = new StreamResult(new StringWriter());
+			tf.transform(new DOMSource(node), dest);
+
+			return dest.getWriter().toString();
 		}
 		catch (Exception e)
 		{
