@@ -1,5 +1,5 @@
 /**
- * $Id: mxCellState.java,v 1.32 2010-06-09 17:32:14 gaudenz Exp $
+ * $Id: mxCellState.java,v 1.33 2011-03-23 13:30:21 gaudenz Exp $
  * Copyright (c) 2007, Gaudenz Alder
  */
 package com.mxgraph.view;
@@ -31,6 +31,12 @@ public class mxCellState extends mxRectangle
 	 */
 	protected Object cell;
 
+	/**
+	 * Holds the current label value, including newlines which result from
+	 * word wrapping.
+	 */
+	protected String label;
+
 	/**
 	 * Contains an array of key, value pairs that represent the style of the
 	 * cell.
@@ -138,6 +144,22 @@ public class mxCellState extends mxRectangle
 	{
 		this.view = view;
 	}
+	
+	/**
+	 * Returns the current label.
+	 */
+	public String getLabel()
+	{
+		return label;
+	}
+	
+	/**
+	 * Returns the current label.
+	 */
+	public void setLabel(String value)
+	{
+		label = value;
+	}
 
 	/**
 	 * Returns the cell that is represented by this state.
