@@ -1,5 +1,5 @@
 /**
- * $Id: mxGraphView.java,v 1.151 2010-08-02 13:44:58 david Exp $
+ * $Id: mxGraphView.java,v 1.154 2011-03-23 13:30:21 gaudenz Exp $
  * Copyright (c) 2007-2010, Gaudenz Alder, David Benson
  */
 package com.mxgraph.view;
@@ -20,8 +20,8 @@ import com.mxgraph.util.mxEventSource;
 import com.mxgraph.util.mxPoint;
 import com.mxgraph.util.mxRectangle;
 import com.mxgraph.util.mxUndoableEdit;
-import com.mxgraph.util.mxUtils;
 import com.mxgraph.util.mxUndoableEdit.mxUndoableChange;
+import com.mxgraph.util.mxUtils;
 import com.mxgraph.view.mxEdgeStyle.mxEdgeStyleFunction;
 import com.mxgraph.view.mxPerimeter.mxPerimeterFunction;
 
@@ -532,6 +532,9 @@ public class mxGraphView extends mxEventSource
 					{
 						updateVertexLabelOffset(state);
 					}
+
+					// Updates the cached label
+					updateLabel(state);
 				}
 			}
 
@@ -730,6 +733,84 @@ public class mxGraphView extends mxEventSource
 		}
 	}
 
+	/**
+	 * Updates the label of the given state.
+	 */
+	public void updateLabel(mxCellState state)
+	{
+		String label = graph.getLabel(state.getCell());
+		Map<String, Object> style = state.getStyle();
+
+		// Applies word wrapping to the label and stores the result in the state
+		if (label != null
+				&& label.length() > 0
+				&& !graph.getModel().isEdge(state.getCell())
+				&& mxUtils.getString(style, mxConstants.STYLE_WHITE_SPACE,
+						"nowrap").equals("wrap"))
+		{
+			double w = getWordWrapWidth(state);
+
+			// The lines for wrapping within the given width are calculated for no
+			// scale. The reason for this is the granularity of actual displayed 
+			// font can cause the displayed lines to change based on scale. A factor 
+			// is used to allow for different overalls widths, it ensures the largest 
+			// font size/scale factor still stays within the bounds. All this ensures
+			// the wrapped lines are constant overing scaling, at the expense the 
+			// label bounds will vary.
+			String[] lines = mxUtils.wordWrap(label,
+					mxUtils.getFontMetrics(mxUtils.getFont(state.getStyle())),
+					w * mxConstants.LABEL_SCALE_BUFFER);
+
+			if (lines.length > 0)
+			{
+				StringBuffer buffer = new StringBuffer();
+
+				for (String line : lines)
+				{
+					buffer.append(line + '\n');
+				}
+
+				label = buffer.substring(0, buffer.length() - 1);
+			}
+		}
+
+		state.setLabel(label);
+	}
+
+	/**
+	 * Returns the width for wrapping the label of the given state at
+	 * scale 1.
+	 */
+	public double getWordWrapWidth(mxCellState state)
+	{
+		Map<String, Object> style = state.getStyle();
+		boolean horizontal = mxUtils.isTrue(style,
+				mxConstants.STYLE_HORIZONTAL, true);
+		double w = 0;
+
+		// Computes the available width for the wrapped label
+		if (horizontal)
+		{
+			w = (state.getWidth() / scale) - 2 * mxConstants.LABEL_INSET - 2
+					* mxUtils.getDouble(style, mxConstants.STYLE_SPACING)
+					- mxUtils.getDouble(style, mxConstants.STYLE_SPACING_LEFT)
+					- mxUtils.getDouble(style, mxConstants.STYLE_SPACING_RIGHT);
+		}
+		else
+		{
+			w = (state.getHeight() / scale)
+					- 2
+					* mxConstants.LABEL_INSET
+					- 2
+					* mxUtils.getDouble(style, mxConstants.STYLE_SPACING)
+					- mxUtils.getDouble(style, mxConstants.STYLE_SPACING_TOP)
+					+ mxUtils
+							.getDouble(style, mxConstants.STYLE_SPACING_BOTTOM);
+		}
+
+		return w;
+	}
+
 	/**
 	 * Updates the label bounds in the given state.
 	 */
@@ -743,13 +824,12 @@ public class mxGraphView extends mxEventSource
 		{
 			state.setLabelBounds(new mxRectangle(state));
 		}
-		else
+		else if (state.getLabel() != null)
 		{
-			String label = graph.getLabel(cell);
 			mxRectangle vertexBounds = (!graph.getModel().isEdge(cell)) ? state
 					: null;
-			state.setLabelBounds(mxUtils.getLabelPaintBounds(label, style,
-					graph.isHtmlLabel(cell), state.getAbsoluteOffset(),
+			state.setLabelBounds(mxUtils.getLabelPaintBounds(state.getLabel(),
+					style, graph.isHtmlLabel(cell), state.getAbsoluteOffset(),
 					vertexBounds, scale));
 		}
 	}
@@ -823,9 +903,8 @@ public class mxGraphView extends mxEventSource
 				double x = state.getX();
 				double y = 0;
 
-				String imgAlign = mxUtils
-						.getString(style, mxConstants.STYLE_IMAGE_ALIGN,
-								mxConstants.ALIGN_CENTER);
+				String imgAlign = mxUtils.getString(style,
+						mxConstants.STYLE_IMAGE_ALIGN, mxConstants.ALIGN_LEFT);
 				String imgValign = mxUtils.getString(style,
 						mxConstants.STYLE_IMAGE_VERTICAL_ALIGN,
 						mxConstants.ALIGN_MIDDLE);
@@ -848,6 +927,7 @@ public class mxGraphView extends mxEventSource
 					y = state.getY() + state.getHeight() - h;
 				}
 				else
+				// MIDDLE
 				{
 					y = state.getY() + (state.getHeight() - h) / 2;
 				}
@@ -861,7 +941,7 @@ public class mxGraphView extends mxEventSource
 		double rotation = mxUtils.getDouble(style, mxConstants.STYLE_ROTATION);
 		mxRectangle bbox = mxUtils.getBoundingBox(rect, rotation);
 
-		// Adda the rotated bounding box to the non-rotated so
+		// Add the rotated bounding box to the non-rotated so
 		// that all handles are also covered
 		rect.add(bbox);
 
