@@ -1,5 +1,5 @@
 /**
- * $Id: mxPerimeter.java,v 1.20 2010-08-09 16:44:11 david Exp $
+ * $Id: mxPerimeter.java,v 1.21 2011-04-12 06:56:22 gaudenz Exp $
  * Copyright (c) 2007-2010, Gaudenz Alder, David Benson
  */
 package com.mxgraph.view;
@@ -155,7 +155,7 @@ public class mxPerimeter
 			double dx = px - cx;
 			double dy = py - cy;
 
-			if (dx == 0)
+			if (dx == 0 && dy != 0)
 			{
 				return new mxPoint(cx, cy + b * dy / Math.abs(dy));
 			}
