@@ -1,5 +1,5 @@
 /**
- * $Id: mxStyleRegistry.java,v 1.4 2010-01-13 10:43:46 gaudenz Exp $
+ * $Id: mxStyleRegistry.java,v 1.6 2011-07-15 12:07:35 gaudenz Exp $
  * Copyright (c) 2007, Gaudenz Alder
  */
 package com.mxgraph.view;
@@ -31,12 +31,14 @@ public class mxStyleRegistry
 		putValue(mxConstants.EDGESTYLE_LOOP, mxEdgeStyle.Loop);
 		putValue(mxConstants.EDGESTYLE_SIDETOSIDE, mxEdgeStyle.SideToSide);
 		putValue(mxConstants.EDGESTYLE_TOPTOBOTTOM, mxEdgeStyle.TopToBottom);
+		putValue(mxConstants.EDGESTYLE_ORTHOGONAL, mxEdgeStyle.orthConnector);
 
 		putValue(mxConstants.PERIMETER_ELLIPSE, mxPerimeter.EllipsePerimeter);
 		putValue(mxConstants.PERIMETER_RECTANGLE,
 				mxPerimeter.RectanglePerimeter);
 		putValue(mxConstants.PERIMETER_RHOMBUS, mxPerimeter.RhombusPerimeter);
 		putValue(mxConstants.PERIMETER_TRIANGLE, mxPerimeter.TrianglePerimeter);
+		putValue(mxConstants.PERIMETER_HEXAGON, mxPerimeter.HexagonPerimeter);
 	}
 
 	/**
